/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appXbase.h"
#include "dlg.h"
#include "trs232ald.h"

#define APPNAME "PIC33pgm"

#define USE_I2C_BRIDGE		0

#define wtyp_TPanelValue	wtyp_USER_Wins
#define wtyp_TPnlI2C_Alg	wtyp_USER_Wins + 100
#define wtyp_TPnlcrt		wtyp_USER_Wins + 200

#define PNLVAL_MAX_DATA		16

#define PNLVAL_AZ_BYTE		0x000000
#define PNLVAL_AZ_WORD		0x010000
#define PNLVAL_AZ_DWORD		0x020000
#define PNLVAL_AZ_INT16		0x030000
#define PNLVAL_AZ_INT32		0x040000
#define PNLVAL_AZ_FLOAT		0x050000
#define PNLVAL_AZ_MASK		0x0f0000

#define TCSelf TPanelValue
typedef struct TPanelValue {
  AlgTWins
  TComboBox *cobValTyp;
  TComboBox *cobSizeTyp;
  TEdit *edAddr;
  TEdit *edData;
  TButton *buRD;
  TButton *buWR;
  TChkBox *cbRDrep;
  TMenuItem *miLastValTyp;
  TMenuItem *miLastSizeTyp;
  dword LastAddr;
  dword Flag;
  int RdDataCount;
  byte RdData[PNLVAL_MAX_DATA];
} TPanelValue;
#undef TCSelf

#define MAX_PNLVAL	6

#define PNLVAL_SENDRD	1
#define PNLVAL_SENDWR	(1<<1)

#define PORTNR_MASK	0x0f
#define PORT_USB_ACM	(1<<8)

#define TCSelf TFormMain
typedef struct TFormMain {
  AlgTForm
  TWins *pnlTop;

  TMenuButton *mbFile;
    TMenu *menuFile;
    TMenuItem *miLoad;
    TMenuItem *miSaveAs;
    TMenuItem *miLcdSaveAs;
  TMenuButton *mbSetting;
    TMenu *menuSetting;
    TMenuItem *miRS232Open;
      TMenu *menuSetDev;
      TMenu *menuSetBaud;
    TMenuItem *miRS232BridgeAuto;
    TMenuItem *miRS232Bridge;
    TMenuItem *mi_ds33E;
    TMenuItem *mi_ds33F;

  TMenuButton *mbHelp;
    TMenu *menuHelp;

  TPanelValue *pnlVal[MAX_PNLVAL];
//  TButton *buOk;

  TPageControl *pgc;
  TPage *pg_dump;
  TPage *pg_pgm;
    char *CPUName;
    TWins *pnlChk;
    TButton *buChkDevice;
    TEdit *edDevice;
    TWins *pnlPgmFile;
    TComboBox *cbPgmFile;
    TButton *buPgmFile;
    TFile_Dlg *dlgPgmFile;

    TButton *buPgmCompFile;
    TButton *buPgmFlashFile;

    TWins *pnlStat;
    TWins *lblStatRun;
    TProgressBar *pbStatProz;
//    TWins *lblStat;
#if USE_I2C_BRIDGE
  TPage *pg_i2c;
    TWins *pnlI2C_Alg;
    TWins *lblI2C_Type;
    TComboBox *cbI2C_Type;
    TWins *lblI2C_Dev;
    TComboBox *cbI2C_Dev;
    TWins *lblI2C_id;
    TComboBox *cbI2C_id;
    TComboBox *cbI2C_id_adr;

    TWins *lblI2C_adr;
    TEdit *edI2C_adr;
    TComboBox *cbI2C_adr;
    TWins *lblI2C_size;
    TEdit *edI2C_size;
#endif
  TPage *pg_crt;

//    int last_Dev;


  TWins *pnlBottom;
  TWins *pnlErr;
  TWins *pnlIOByte;
  TWins *pnlCpuType;

  TFile_Dlg *File_Dlg;
  TFile_Dlg *FileSave_Dlg;
  TFile_Dlg *FileSaveLCD_Dlg;

  Trs232ald *rs232;
  TgTimer *rs232Timer;
  int ErrTimeOut, ErrTimeOutRep, ErrCrc;
  int TimeSec;
  bool HasChkProgrammer;
  bool IsProgrammer;
  bool BridgeModus;
  bool BridgeModusMk;
  bool BridgeAuto;
} TFormMain;
#undef TCSelf

void MenuFile_Create(TFormMain *self);

extern int step;
extern TSysInfo0 CPU_Info0;


#define STEP_SYNC	0
#define STEP_CHKPROG	1
#define STEP_INFO_0	2
#define STEP_PNLVAL	3
#define STEP_PGM	4
#define STEP_I2C	5
#define STEP_CRT	6
#define STEP_BRIDGE	7

void rs232ald_ErrPrint(dword orig_bef, dword bef, const byte *data, size_t lg);
Trs232ald *Get_rs232();

void init_pg_pgm(TPage *self);

void set_pg_pgm_dev(TFormMain *mf);

bool OnRxDData_pg_ext(Trs232ald *self, dword bef, const byte *data, size_t lg);
bool run_pg_ext(TFormMain *mf);
void set_en_pg_pgm(TFormMain *mf, bool en);


#define RSBEF_SETCPUID		RSBEF_USER
#define RSBEF_CPU_RESET		RSBEF_USER + 1
#define RSBEF_ICSP		RSBEF_USER + 2
#define RSBEF_SendICSP_BEF	RSBEF_USER + 3
#define RSBEF_ExecICSP_BEF	RSBEF_USER + 4


#if USE_I2C_BRIDGE
void init_pg_i2c(TPage *self);
void pg_i2c_LoadFile(TFormMain *mf);
void pg_i2c_SaveFile(TFormMain *mf);

void set_en_pg_i2c(TFormMain *self, bool en);
bool start_pg_i2c(TFormMain *self);
bool OnRxDData_pg_i2c(Trs232ald *self, dword bef, const byte *data, size_t lg);
#endif

void init_pg_crt(TPage *self);
void set_pg_crt(TSysInfo0 *SysInfo0);
void set_en_pg_crt(TFormMain *self, bool en);
bool run_pg_crt(TFormMain *self);
bool OnRxDData_pg_crt(Trs232ald *self, dword bef, const byte *data, size_t lg);
bool save_as_pg_crt_ok(TFormMain *self);
void pg_crt_SaveFile(TFormMain *mf);
