/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "PIC33pgm.h"

typedef struct {
  const dword count;
  const dword *data;
} TICSP_bef;

//static byte TxData[256];

#define SETADR24	0x01
#define SETCOUNT8	0x02
#define LOOP_COUNT8	0x03
#define REGOUT		0x08


#define InitTBLPAG_W6	0x80
static const dword DE_InitTBLPAG_W6[3] = {
  0x00200000,		//MOV #<SourceAddress23:16>, W0
  0x008802A0,		//MOV W0, TBLPAG
  0x00200006		//MOV #<SourceAddress15:0>, W6
};

static const dword DE_NOP_0x81[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x82[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x83[1] = {
  0x00000000		//nop
};

static const dword DE_NOP_0x84[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x85[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x86[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x87[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x88[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x89[1] = {
  0x00000000		//nop
};



#define ExitReset	0x8a
static const dword DE_ExitResetVector[2] = {
  0x02000000,		//NOP				+2 NOP
  0x03040200		//GOTO 0x200			+3 NOP
};

//#define ResetDevicePC	0x8b
static const dword DE_ResetDevicePC[2] = {
  0x02000000,		//NOP				+2 NOP
  0x03040200		//GOTO 0x200			+3 NOP
};


#define VISI_W7		0x8c
static const dword DE_InitVISI_W7[1] = {
  0x0120F887		//MOV #VISI, W7			+1 NOP
};

#define ReadData_1word	0x8d
static const dword DE_ReadData_1word[1] = {
  0x05BA0BB6		//TBLRDL [W6++], [W7]		+5 NOP
};


static const dword DE_ReadData_6word[9] = {
  0x01EB0380,		// CLR W7			+ 1*NOP
  0x05BA1B96,		// TBLRDL [W6], [W7++]		+ 5*NOP
  0x05BADBB6,		// TBLRDH.B [W6++], [W7++]	+ 5*NOP
  0x05BADBD6,		// TBLRDH.B [++W6], [W7++]	+ 5*NOP
  0x05BA1BB6,		// TBLRDL [W6++], [W7++]	+ 5*NOP
  0x05BA1B96,		// TBLRDL [W6], [W7++]		+ 5*NOP
  0x05BADBB6,		// TBLRDH.B [W6++], [W7++]	+ 5*NOP
  0x05BADBD6,		// TBLRDH.B [++W6], [W7++]	+ 5*NOP
  0x05BA0BB6,		// TBLRDL [W6++], [W7]		+ 5*NOP
};




#define MAX_ICSP_BEF	16

static const TICSP_bef ICSP_bef[MAX_ICSP_BEF] = {
  {3,  DE_InitTBLPAG_W6},
  {1,  DE_NOP_0x81},
  {1,  DE_NOP_0x82},
  {1,  DE_NOP_0x83},
  {1,  DE_NOP_0x84},
  {1,  DE_NOP_0x85},
  {1,  DE_NOP_0x86},
  {1,  DE_NOP_0x87},
  {1,  DE_NOP_0x88},
  {1,  DE_NOP_0x89},
  {2,  DE_ExitResetVector},
  {2,  DE_ResetDevicePC},
  {1,  DE_InitVISI_W7},
  {1,  DE_ReadData_1word},
  {9,  DE_ReadData_6word},
  {0, 0}
};


int Get_dsPIC32E_ICSP_bef(int *idx, byte *data) {
  int i;
  const TICSP_bef *bef;

	i = *idx;
	if (i < 0) i = 0;
	if (i >= MAX_ICSP_BEF) return 0;

	bef = ICSP_bef + i;
	if (!bef->count || !bef->data) return 0;
	i++;
	*idx = i;

	data[0] = 2;
	data[1] = bef->count;
	pnmove(data + 2, bef->data, bef->count * 4);

	return (bef->count * 4) + 2;
}

/*  Read n Word
0x01		Init Pointer
0xll
0xmm
0xhh
0x02		Count
0xnn

0x80		ExitResetVector
0x10		InitializeTBLPAG_W7_VISI(addr);	    
0x82		DE_ReadData_1word		    <---#
0x11		REGOUT					|
0x03		loop Count > 0				|
0x03						    >---+
0x81		DE_ResetDevicePC
*/

static const byte DE_Read_nWord[] = {
  SETADR24,			//Adr   +24bit
  SETCOUNT8,			//Count +8bit  1..200
  ExitReset,
  InitTBLPAG_W6,
  VISI_W7,			//	<---+
  ReadData_1word,		//	    |
  REGOUT,			//	    |
  LOOP_COUNT8, 3,		//	->--+
  0
};

int Get_dsPIC32E_Read_bef(byte *data, dword adr, dword len) {
  int result;
  const byte *bef;

	if (!len || len >= 120) return 0;
	result = 0;
	bef = DE_Read_nWord;
	while (*bef) {
//	for (i=sizeof(DE_Read_nWord);i>0;i--) {
	    *data = *bef;
	    data++; result++;
	    if (*bef == SETADR24) {
		*data = adr;	   data++; result++;
		*data = adr >> 8;  data++; result++;
		*data = adr >> 16; data++; result++;

	    } else if (*bef == SETCOUNT8) {
		*data = len - 1; data++; result++;

	    } else if (*bef == LOOP_COUNT8) {
		bef++;
		*data = *bef;
		data++; result++;
	    }
	    bef++;
	}
 printf("%s(%d): %d\n", FL, result);
	return result;
}
