/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "PIC33pgm.h"
#include "dlg.h"
#include "cpu_pgm_ds33E.h"

#define MAX_PUSH_DATA 10

static bool RunReset, RunOpen, RunRead, CpuPgmStart;

static TCpuPgm *CpuPgm;


/* ______________________________________________________________________ */
static void buPgmFile_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *mf;
  char *s, *sg;
  const char *fn;
  TpcIni *ini;

	mf = appDat.obj;
	if (File_Dlg_Execute(mf->dlgPgmFile)) {
	    s = NULL;
	    if (File_Dlg_GetEdFileName(mf->dlgPgmFile, &s)) {
		fn = path_korr_file(s);
		SetCaption((TWins*)mf->cbPgmFile, fn);
		TComboBox_Push(mf->cbPgmFile, MAX_PUSH_DATA);
		if (mf->CPUName) {
		    ini = get_appIni();
		    sg = pcadd("PGM_", mf->CPUName);
		    TpcIni_push_stack(ini, sg, "LastFile", fn, MAX_PUSH_DATA, 0);
		    xfree(sg);
		    free_appIni(ini);
		}
		xfree(s);
	    }
	}
}

static void Callback_Path(TpcIni *self, const char *Data, void *owner) {
	TComboBox_AddItem(owner, Data, 0, NULL);
}

static void cbPgmFile_OnItemSel(TComboBox *self, TMenuItem *SelItem) {
  TFormMain *mf;
  char *sg;
  TpcIni *ini;

	mf = appDat.obj;
	if (mf->CPUName) {
	    ini = get_appIni();
	    sg = pcadd("PGM_", mf->CPUName);
	    TpcIni_push_stack(ini, sg, "LastFile", GetCaption_pc((TWins*)self), MAX_PUSH_DATA, 0);
	    xfree(sg);
	    free_appIni(ini);
	}
}

static void pnlPgmFile_OnResize(TWins* self) {
  TFormMain *mf;
  int w;

	mf = appDat.obj;
	w = self->width - mf->cbPgmFile->left - 2;
	if (w < 100) w = 100;
	SetWidth((TWins*)mf->cbPgmFile, w);
}
/* ______________________________________________________________________ */
static void buChkDevice_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *mf;

	mf = appDat.obj;
	TCpuPgm_open(CpuPgm, mf->rs232);
	TCpuPgm_close(CpuPgm, false);
	CpuPgmStart = true;
}


static void pnlChk_OnResize(TWins* self) {
  TFormMain *mf;
  int w;

	mf = appDat.obj;
	w = self->width - mf->edDevice->left - 17;
	if (w < 100) w = 100;
	SetWidth((TWins*)mf->edDevice, w);
}

/* ______________________________________________________________________ */
static void buPgmCompFile_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *mf;
  const char *fn;

	mf = appDat.obj;
	fn = GetCaption_pc((TWins*)mf->cbPgmFile);
	if (!fn || !*fn) return;
// printf("%s(%d): %s\n", FL, fn);

//	if (CpuPgm && CpuPgm->HexFileIO) CpuPgm->HexFileIO->debug = true;

	if (TCpuPgm_LoadFromFile(CpuPgm, fn)) {
	    TCpuPgm_open(CpuPgm, mf->rs232);
	    TCpuPgm_comp_mem(CpuPgm);
	    TCpuPgm_close(CpuPgm, false);
	    CpuPgmStart = true;
	}
}
/* ______________________________________________________________________ */
static void buPgmFlashFile_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *mf;
  const char *fn;

	mf = appDat.obj;
//	fn = "/root/pic30/PICpgm/PICpgm.hex";
	fn = GetCaption_pc((TWins*)mf->cbPgmFile);
	if (!fn || !*fn) return;
 printf("%s(%d): %s\n", FL, fn);


	if (CpuPgm && CpuPgm->HexFileIO) CpuPgm->HexFileIO->debug = true;

	if (TCpuPgm_LoadFromFile(CpuPgm, fn)) {
	    TCpuPgm_open(CpuPgm, mf->rs232);
	    TCpuPgm_flash_mem(CpuPgm);
	    TCpuPgm_close(CpuPgm, false);
	    CpuPgmStart = true;
	}
}
/* ______________________________________________________________________ */
static void CpuPgm_OnDataRead(TCpuPgm *self, byte *data, dword adr, size_t len) {
#if 0
  int i, lg;
  word *pw;

	lg = len;
	pw = (word*)data;
	for (i=0;i<4;i++) {
	    if (lg < 16) break;
	    printf("0x%06x  0x%04x 0x%04x 0x%04x 0x%04x  0x%04x 0x%04x 0x%04x 0x%04x\n", adr + (i * 0x10), pw[0], pw[1], pw[2], pw[3], pw[4], pw[5], pw[6], pw[7]);
	    pw += 8;
	    lg -= 16;
	}
#endif
}

static void CpuPgm_OnDeviceID(TCpuPgm *self, const char *DevStr, const char *CPUName) {
  TFormMain *mf;
  char *sg;
  TpcIni *ini;

	mf = appDat.obj;
	SetCaption((TWins*)mf->edDevice, DevStr);
	if (CPUName) {
	    SetEnabled((TWins*)mf->pnlPgmFile, true);
	    SetEnabled((TWins*)mf->buPgmCompFile, true);
	    SetEnabled((TWins*)mf->buPgmFlashFile, true);
	    SetCaption((TWins*)mf->lblStatRun, "OK");
	    TProgressBar_SetProz(mf->pbStatProz, 0);
	    if (!pccmp(mf->CPUName, CPUName)) {
		ppcchange(&mf->CPUName, CPUName);
		TComboBox_ClsMenu(mf->cbPgmFile);

		ini = get_appIni();
		sg = pcadd("PGM_", mf->CPUName);
		TpcIni_get_stack(ini, sg, "LastFile", Callback_Path, mf->cbPgmFile, 
				MAX_PUSH_DATA, 0);
		xfree(sg);
		free_appIni(ini);
		TComboBox_SetItem(mf->cbPgmFile, 0);

	    }
	} else {
	    ppcfree(&mf->CPUName);
	    TComboBox_ClsMenu(mf->cbPgmFile);
	    SetCaption((TWins*)mf->cbPgmFile, NULL);
	    SetEnabled((TWins*)mf->pnlPgmFile, false);
	}
}

static void CpuPgm_OnClose(TCpuPgm *self, int err) {
	if (err) appError(EMSG_ERR1, self->Errbuf);
}
static bool CpuPgm_OnError(TCpuPgm *self, const char *s) {
	printf("ERROR: %s\n", s);
	return true;
}
static void CpuPgm_OnStatus(TCpuPgm *self, int Stat, int StatuFu) {
  TFormMain *mf;
  int proz;

	proz = 0;
	if (self->GesDataSize > 0) {
	    proz = self->NowDataSize * 1000 / self->GesDataSize;
	}

	mf = appDat.obj;
	if (StatuFu & STATFU_ERROR) {
	    SetColor((TWins*)mf->lblStatRun, 0xF08080);
	} else if (Stat == STAT_OPEN)  {
	    SetColor((TWins*)mf->lblStatRun, 0xC0C0C0);
	}

	if (Stat == STAT_OPEN) {
	    SetCaption((TWins*)mf->lblStatRun, "OPEN");
	    TProgressBar_SetProz(mf->pbStatProz, 0);

	} else if (Stat == STAT_CLR_FLASH) {
	    SetCaption((TWins*)mf->lblStatRun, "CLR FLASH");

	} else if (Stat == STAT_COMPARE) {
	    if (StatuFu & STATFU_ERROR) {
		SetCaption((TWins*)mf->lblStatRun, "COMPARE FLASH ERROR");
	    } else if (StatuFu == STATFU_END) {
		SetCaption((TWins*)mf->lblStatRun, "COMPARE FLASH OK");
		TProgressBar_SetProz(mf->pbStatProz, 1000);
	    } else {
		SetCaption((TWins*)mf->lblStatRun, "COMPARE FLASH");
		TProgressBar_SetProz(mf->pbStatProz, proz);
	    }

	} else if (Stat == STAT_PROGRAMING) {
	    SetCaption((TWins*)mf->lblStatRun, "PROGRAMING");
	    TProgressBar_SetProz(mf->pbStatProz, proz);

//	} else {
// printf("Stat=%d StatuFu=%d %d %d\n", Stat, StatuFu, self->GesDataSize, self->NowDataSize);
	}
}

void init_pg_pgm(TPage *self) {
  TFormMain *mf;

	mf = appDat.obj;
	mf->pnlChk = TWins_CreateBA((TWins*)self, 0, 0, 200, 22, 0, alTop);
	mf->pnlChk->OnResize = pnlChk_OnResize;
	mf->buChkDevice = TButton_CreateS(mf->pnlChk, 1, 1, 80, 20, "Chk Dev.");
	mf->buChkDevice->OnClick = buChkDevice_OnClick;
	SetEnabled((TWins*)mf->buChkDevice, false);
	mf->edDevice = TEdit_CreateS(mf->pnlChk, 83, 1, 300, 20, " --- ");
	mf->edDevice->ReadOnly = true;

	mf->pnlPgmFile = TWins_CreateBA((TWins*)self, 0, 0, 200, 22, 0, alTop);
	mf->pnlPgmFile->OnResize = pnlPgmFile_OnResize;
	mf->cbPgmFile = TComboBox_CreateS(mf->pnlPgmFile, 83, 1, 300, 20, NULL);
	mf->cbPgmFile->OnItemSel = cbPgmFile_OnItemSel;
	mf->buPgmFile = TButton_CreateS(mf->pnlPgmFile, 1, 1, 80, 20, "File");
	mf->buPgmFile->OnClick = buPgmFile_OnClick;
	mf->dlgPgmFile = Create_File_Dlg((TWins*)mf);
	File_Dlg_AddPattern(mf->dlgPgmFile, "*.hex", NULL);
	SetEnabled((TWins*)mf->pnlPgmFile, false);

	mf->buPgmCompFile = TButton_CreateS((TWins*)self, 1, 62, 80, 20, "Comp File");
	mf->buPgmCompFile->OnClick = buPgmCompFile_OnClick;
	SetEnabled((TWins*)mf->buPgmCompFile, false);

	mf->buPgmFlashFile = TButton_CreateS((TWins*)self, 100, 62, 80, 20, "Flash File");
	mf->buPgmFlashFile->OnClick = buPgmFlashFile_OnClick;
	SetEnabled((TWins*)mf->buPgmFlashFile, false);

	mf->pnlStat = TWins_CreateBA((TWins*)self, 0, 0, 200, 42, mBorder, alBottom);
	mf->lblStatRun = TWins_CreateLabel(mf->pnlStat, 2, 2, 200, 18, NULL, 0, mBorder|mBorderL);
	mf->pbStatProz = TProgressBar_CreateU(mf->pnlStat, 2, 21, 200, 18);

	set_pg_pgm_dev(mf);
}

void set_pg_pgm_dev(TFormMain *mf) {
	if (mf->mi_ds33F->fu & miChecked) {
	    if (CpuPgm && (CpuPgm->dsFL_33 == FL_33F)) return;
	} else {
	    if (CpuPgm && (CpuPgm->dsFL_33 == FL_33E)) return;
	}
	if (CpuPgm) Tfree(CpuPgm);

	if (mf->mi_ds33F->fu & miChecked) {
	    CpuPgm = TCpuPgm_dsPIC33F_Create();
	} else {
	    CpuPgm = TCpuPgm_dsPIC33E_Create();
	}

	CpuPgm->OnDataRead = CpuPgm_OnDataRead;
	CpuPgm->OnDeviceID = CpuPgm_OnDeviceID;
	CpuPgm->OnClose = CpuPgm_OnClose;
	CpuPgm->OnError = CpuPgm_OnError;
	CpuPgm->OnStatus = CpuPgm_OnStatus;
}

void set_en_pg_pgm(TFormMain *mf, bool en) {
	SetEnabled((TWins*)mf->buChkDevice, en);
	if (!en) {
	    SetEnabled((TWins*)mf->buPgmCompFile, false);
	    SetEnabled((TWins*)mf->buPgmFlashFile, false);
	    CpuPgmStart = false;
	}
}

bool run_pg_ext(TFormMain *mf) {
	if (CpuPgmStart) {
	    CpuPgmStart = false;
	    CpuPgm->BridgeModus = mf->BridgeModus;
	    if (TCpuPgm_NextTxDData(CpuPgm)) return true;
	}
	return false;
}

bool OnRxDData_pg_ext(Trs232ald *self, dword bef, const byte *data, size_t lg) {

	CpuPgmStart = false;
	RunReset = false;
	RunOpen = false;
	RunRead = false;

	if (TCpuPgm_RxDData(CpuPgm, bef, data, lg)) {
// printf("%s(%d): 0x%02x  lg=%d  0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", FL, bef, lg, data[0], data[1], data[2], data[3], data[4], data[5]);
	    if (TCpuPgm_NextTxDData(CpuPgm)) return true;
	    step = STEP_SYNC;
	}
	return false;
}
