/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appbase.h"
#include "cpu_pgm.h"

bool TCpuPgm_LoadFromFile(TCpuPgm *self, const char *FileName) {
	if (!self->HexFileIO) return false;
	return THexFileIO_LoadFromFile(self->HexFileIO, FileName);
}

bool TCpuPgm_NextTxDData(TCpuPgm *self) {
	if (!self || !self->rs232ald || !self->NextTxDData) return false;
	return self->NextTxDData(self);
}

bool TCpuPgm_RxDData(TCpuPgm *self, dword bef, const byte *data, size_t len) {
	if (!self || !self->rs232ald || !self->RxDData) return false;
	return self->RxDData(self, bef, data, len);
}


bool TCpuPgm_open(TCpuPgm *self, Trs232ald *rs232ald) {
	if (!self || !rs232ald || !self->open) return false;
	return self->open(self, rs232ald);
}

bool TCpuPgm_close(TCpuPgm *self, bool force) {
	if (!self || !self->close) return false;
	if (!self->rs232ald) return true;
	return self->close(self, force);
}

bool TCpuPgm_read(TCpuPgm *self, dword adr, size_t len) {
	if (!self || !len || !self->read) return false;
	return self->read(self, adr, len);
}

bool TCpuPgm_read_prog_mem(TCpuPgm *self, dword adr, size_t len) {
	if (!self || !len || !self->read_prog_mem) return false;
	return self->read_prog_mem(self, adr, len);
}

bool TCpuPgm_flash_mem(TCpuPgm *self) {
	if (!self || !self->HexFileIO  || !self->HexFileIO->Seg0 || 
	    !self->flash_mem) return false;
	return self->flash_mem(self);
}

bool TCpuPgm_comp_mem(TCpuPgm *self) {
	if (!self || !self->HexFileIO  || !self->HexFileIO->Seg0 || 
	    !self->comp_mem) return false;
	return self->comp_mem(self);
}

bool TCpuPgm_cpureset(TCpuPgm *self, Trs232ald *rs232ald, bool force) {
	if (!self || !rs232ald || !self->cpureset) return false;
	return self->cpureset(self, rs232ald, force);
}


/*_____________________________ Destroy _________________________________*/
bool TCpuPgm__Destroy(TCpuPgm *self) {
	self->rs232ald = NULL;
	xfree(self->RxDbuf);
	xfree(self->TxDbuf);
	xfree(self->IObuf);
	Tfree(self->HexFileIO);
	return false;
}
/*_____________________________ Create __________________________________*/
void *TCpuPgm_Create_struct_init(int tcsize, int wtyp) {
  TCpuPgm *self;

	self = ttc_create(tcsize, wtyp);
	self->destroy = (TDESTROY)TCpuPgm__Destroy;
	self->HexFileIO = THexFileIO_Create();
/*	self->open = Trs232_open;
	self->close = Trs232_close;
	self->tc_private = xcalloc(sizeof(ttc_private));
	self->RxDsize = RS232MAXBUF;
	self->RxDbuf = xmalloc(self->RxDsize); */
	return self;
}

/* TCpuPgm *TCpuPgm_Create() {
	return TCpuPgm_Create_struct_init(sizeof(TCpuPgm), wtyp_Trs232);
} */
