/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef __CPUPGM_H__
#define __CPUPGM_H__

#include "trs232ald.h"
#include "thexfileio.h"


#define ds33BEF_SETCPUID		RSBEF_USER
#define ds33BEF_CPU_RESET		RSBEF_USER + 1
#define ds33BEF_ICSP			RSBEF_USER + 2
#define ds33BEF_SendICSP_BEF		RSBEF_USER + 3
#define ds33BEF_ExecICSP_BEF		RSBEF_USER + 4
#define ds33BEF_SEND_DATA		RSBEF_USER + 5

#define STAT_OPEN		1
#define STAT_CLR_FLASH		2
#define STAT_READ		3
#define STAT_COMPARE		4
#define STAT_PROGRAMING		5
#define STAT_ERROR		8
#define STAT_CLOSE		9

#define STATFU_NONE		0
#define STATFU_START		1
#define STATFU_RUN		2
#define STATFU_END		3
#define STATFU_ERROR		0x80000000



#define AlgTCpuPgm    AlgTTC \
  byte *IObuf; \
  dword IO_size; \
  dword read_start_adr; \
  dword read_len; \
  dword read_adr; \
  dword write_len; \
  dword write_adr; \
  dword Start_ms; \
  dword GesDataSize; \
  dword NowDataSize; \
  int dsFL_33; \
  bool BridgeModus; \
  bool cb_bnc1, cb_bnc2, cb_bnc3; \
  bool (*open)(struct TCSelf *self, Trs232ald *rs232ald); \
  bool (*close)(struct TCSelf *self, bool force); \
  bool (*read)(struct TCSelf *self, dword adr, size_t len); \
  bool (*read_prog_mem)(struct TCSelf *self, dword adr, size_t len); \
  bool (*comp_mem)(struct TCSelf *self); \
  bool (*flash_mem)(struct TCSelf *self); \
  bool (*cpureset)(struct TCSelf *self, Trs232ald *rs232ald, bool force); \
  bool (*RxDData)(struct TCSelf *self, dword bef, const byte *data, size_t len); \
  bool (*NextTxDData)(struct TCSelf *self); \
  void (*OnDataRead)(struct TCSelf *self, byte *data, dword adr, size_t len); \
  void (*OnDataSend)(struct TCSelf *self); \
  void (*OnDeviceID)(struct TCSelf *self, const char *DevStr, const char *CPUName); \
  void (*OnClose)(struct TCSelf *self, int err); \
  bool (*OnError)(struct TCSelf *self, const char *s); \
  void (*OnStatus)(struct TCSelf *self, int Stat, int StatuFu); \
  int ErrorCount; \
  byte *RxDbuf; \
  byte *TxDbuf; \
  size_t IObuf_size; \
  Trs232ald *rs232ald; \
  THexFileIO *HexFileIO; \
  TSegment *HexIOSeg; \
  int HexIOSegOffset; \
  int AktError; \
  char Errbuf[120];

#define TCSelf TCpuPgm
typedef struct TCpuPgm {
  AlgTCpuPgm
} TCpuPgm;
#undef TCSelf


void *TCpuPgm_Create_struct_init(int tcsize, int wtyp);
bool TCpuPgm__Destroy(TCpuPgm *self);

bool TCpuPgm_open(TCpuPgm *self, Trs232ald *rs232ald);
bool TCpuPgm_close(TCpuPgm *self, bool force);
bool TCpuPgm_read(TCpuPgm *self, dword adr, size_t len);
bool TCpuPgm_read_prog_mem(TCpuPgm *self, dword adr, size_t len);
bool TCpuPgm_comp_mem(TCpuPgm *self);
bool TCpuPgm_flash_mem(TCpuPgm *self);
bool TCpuPgm_cpureset(TCpuPgm *self, Trs232ald *rs232ald, bool force);
bool TCpuPgm_RxDData(TCpuPgm *self, dword bef, const byte *data, size_t len);
bool TCpuPgm_NextTxDData(TCpuPgm *self);

bool TCpuPgm_LoadFromFile(TCpuPgm *self, const char *FileName);

#endif /* __CPUPGM_H__ */
