/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appbase.h"
#include "cpu_pgm_ds33E.h"

//#define PGMSTEP_CPU_RESET	2
//#define PGMSTEP_CPU_START	3
#define EFLAG_PROGRAMMER_ON	1
#define EFLAG_SETCPUID		(1<<1)
#define EFLAG_EnterICSP		(1<<2)
#define EFLAG_ClrICSP_BEF	(1<<3)
#define EFLAG_SendICSP_BEF	(1<<4)
#define EFLAG_ReadDevID		(1<<5)
#define EFLAG_ClrFLASH		(1<<6)
#define EFLAG_READ_N_WORD	(1<<7)
#define EFLAG_READ_N_PROGMEM	(1<<8)
//#define EFLAG_WRITE_N_WORD	(1<<8)
#define EFLAG_WRITE_N_PROGMEM	(1<<10)
#define EFLAG_FLASH_PROGMEM	(1<<11)
#define EFLAG_COMP_PROGMEM	(1<<12)


#define EFLAG_ExitICSP		(1<<30)
#define EFLAG_PROGRAMMER_OFF	(1<<31)

#define FLAGS_OPEN	EFLAG_PROGRAMMER_ON|EFLAG_SETCPUID|EFLAG_EnterICSP|\
			EFLAG_ClrICSP_BEF|EFLAG_SendICSP_BEF|EFLAG_ReadDevID


typedef struct {
  const dword count;
  const dword *data;
} TICSP_bef;

//static byte TxData[256];

#define SETADR24		0x01
#define SETCOUNT8		0x02
#define SET_MAX_COUNT8		0x03
#define LOOP_COUNT8		0x04
#define LOOP_MAX_WaitPgm	0x05
#define Skip_FF_2x24Bit		0x06

#define DELAY_N10US		0x08
#define DELAY_N100US		0x09
#define REGOUT_L		0x0a
#define REGOUT_H		0x0b

//#define REGOUT8_TxD	0x0c
#define REGOUT16_TxD	0x0d
#define REGOUT24_TxD	0x0e
//#define REGOUT32_TxD	0x0f

//#define REGOUT8_TxD	0x10
//#define REGOUT16_TxD	0x11
#define REGOUT24_COMP	0x12
//#define REGOUT32_TxD	0x13


#define InitTBLPAG_W6	0x80
static const dword DE_InitTBLPAG_W6[3] = {
  0x00200000,		//MOV #<SourceAddress23:16>, W0
  0x008802A0,		//MOV W0, TBLPAG
  0x00200006		//MOV #<SourceAddress15:0>, W6
};

#define Load_2x24Bit	0x81
static const dword DE_Load_2x24Bit[3] = {
  0x200000,		//MOV #<LSW0>, W0
  0x200001,		//MOV #<MSB1:MSB0>, W1
  0x200002		//MOV #<LSW1>, W2
};

#define InitAdr_2xReg_W3W4	0x82
static const dword DE_InitAdr_2xReg_W3W4[2] = {
  0x200003,		//MOV #<DestinationAddress15:0>, W3
  0x200004		//MOV #<DestinationAddress23:16>, W4
};
static const dword DE_NOP_0x83[1] = {
  0x00000000		//nop
};

static const dword DE_NOP_0x84[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x85[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x86[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x87[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x88[1] = {
  0x00000000		//nop
};
static const dword DE_NOP_0x89[1] = {
  0x00000000		//nop
};



#define ExitReset	0x8a
static const dword DE_ExitResetVector[2] = {
  0x02000000,		//NOP				+2 NOP
  0x03040200		//GOTO 0x200			+3 NOP
};

//#define ResetDevicePC	0x8b
static const dword DE_ResetDevicePC[2] = {
  0x02000000,		//NOP				+2 NOP
  0x03040200		//GOTO 0x200			+3 NOP
};


#define VISI_W7		0x8c
static const dword DE_InitVISI_W7[1] = {
  0x0120F887		//MOV #VISI, W7			+1 NOP
};

#define ReadData_1word	0x8d
static const dword DE_ReadData_1word[1] = {
  0x05BA0BB6		//TBLRDL [W6++], [W7]		+5 NOP
};

#define ReadDataL_word	0x8e
static const dword DE_ReadDataL_word[1] = {
  0x05BA0B96		//TBLRDL [W6], [W7]		+5 NOP
};
#define ReadDataH_word	0x8f
static const dword DE_ReadDataH_word[1] = {
  0x05BA8BB6		//TBLRDH [W6++], [W7]		+5 NOP
};

#define EraseAllMemory	0x90
static const dword DE_EraseAllMemory[2] = {
  0x002400DA,		//MOV #0x400D, W10
  0x0288394A		//MOV W10, NVMCON		+2 NOP
};

#define SetWR_Flash	0x91
static const dword DE_SetWR_Flash[5] = {
  0x00200551,		//MOV #0x55, W1
  0x00883971,		//MOV W1, NVMKEY
  0x00200AA1,		//MOV #0xAA, W1
  0x00883971,		//MOV W1, NVMKEY
  0x05A8E729		//BSET NVMCON, #WR		+5 NOP
};

#define WaitProgramOperation	0x92
static const dword DE_WaitProgramOperation[3] = {
  0x01000000,		//NOP				+1 NOP
  0x01803940,		//MOV NVMCON, W0		+1 NOP
  0x01887C40		//MOV W0, VISI			+1 NOP
};

#define Init_FA_TBLPAG		0x93
static const dword DE_Init_FA_TBLPAG[2] = {
  0x00200FAC,		//MOV #0xFA, W12
  0x008802AC		//MOV W12, TBLPAG
};

#define FLASH_Prog2wr		0x94
static const dword D_FLASH_Prog2wr[6] = {
  0x01EB0300,		//CLR W6			+1 NOP
  0x01EB0380,		//CLR W7			+1 NOP
  0x02BB0BB6,		//TBLWTL[W6++], [W7]		+2 NOP
  0x02BBDBB6,		//TBLWTH.B[W6++], [W7++]	+2 NOP
  0x02BBEBB6,		//TBLWTH.B[W6++], [++W7]	+2 NOP
  0x02BB1BB6		//TBLWTL[W6++], [W7++]		+2 NOP
};

#define W3W4_TO_NVMADR		0x95
static const dword DE_W3W4_TO_NVMADR[2] = {
  0x00883953,		//MOV W3, NVMADR
  0x00883964		//MOV W4, NVMADRU
};

#define FLASH_ProgOpen		0x96
static const dword DE_FLASH_ProgOpen[2] = {
  0x0124001A,		//MOV #0x4001, W10		+1 NOP
  0x0288394A,		//MOV W10, NVMCON		+2 NOP
};

/* static const dword DE_ReadData_6word[9] = {
  0x01EB0380,		// CLR W7			+ 1*NOP
  0x05BA1B96,		// TBLRDL [W6], [W7++]		+ 5*NOP
  0x05BADBB6,		// TBLRDH.B [W6++], [W7++]	+ 5*NOP
  0x05BADBD6,		// TBLRDH.B [++W6], [W7++]	+ 5*NOP
  0x05BA1BB6,		// TBLRDL [W6++], [W7++]	+ 5*NOP
  0x05BA1B96,		// TBLRDL [W6], [W7++]		+ 5*NOP
  0x05BADBB6,		// TBLRDH.B [W6++], [W7++]	+ 5*NOP
  0x05BADBD6,		// TBLRDH.B [++W6], [W7++]	+ 5*NOP
  0x05BA0BB6,		// TBLRDL [W6++], [W7]		+ 5*NOP
}; */



static const TICSP_bef ICSP_bef[] = {
  {3,  DE_InitTBLPAG_W6},
  {3,  DE_Load_2x24Bit},
  {2,  DE_InitAdr_2xReg_W3W4},
  {1,  DE_NOP_0x83},
  {1,  DE_NOP_0x84},
  {1,  DE_NOP_0x85},
  {1,  DE_NOP_0x86},
  {1,  DE_NOP_0x87},
  {1,  DE_NOP_0x88},
  {1,  DE_NOP_0x89},
  {2,  DE_ExitResetVector},
  {2,  DE_ResetDevicePC},
  {1,  DE_InitVISI_W7},
  {1,  DE_ReadData_1word},
  {1,  DE_ReadDataL_word},
  {1,  DE_ReadDataH_word},
  {2,  DE_EraseAllMemory},	//EraseAllMemory	0x90
  {5,  DE_SetWR_Flash},		//SetWR_Flash		0x91
  {3,  DE_WaitProgramOperation},//WaitProgramOperation	0x92
  {3,  DE_Init_FA_TBLPAG},	//Init_FA_TBLPAG	0x93
  {6,  D_FLASH_Prog2wr},	//FLASH_Prog2wr		0x94
  {2,  DE_W3W4_TO_NVMADR},	//W3W4_TO_NVMADR	0x95
  {2,  DE_FLASH_ProgOpen},	//FLASH_ProgOpen	0x96
  {0, 0}
};


static const byte SQ_Read_nWord[] = {
  SETADR24,			//Adr   +24bit
  SETCOUNT8,			//Count +8bit  1..200
  ExitReset,
  InitTBLPAG_W6,
  VISI_W7,			//	
  ReadData_1word,		//	<---+
  REGOUT_L,			//	    |
  REGOUT16_TxD,			//	    |
  LOOP_COUNT8, 3,		//	->--+
  0
};

static const byte SQ_Read_nProgMem[] = {
  SETADR24,			//Adr   +24bit
  SETCOUNT8,			//Count +8bit  1..200
  ExitReset,
  InitTBLPAG_W6,
  VISI_W7,			//	
  ReadDataL_word,		//	<---+
  REGOUT_L,			//	    |
  ReadDataH_word,		//	    |
  REGOUT_H,			//	    |
  REGOUT24_TxD,			//	    |
  LOOP_COUNT8, 5,		//	->--+
  0
};

static const byte SQ_EraseAllMemory[] = {
  ExitReset,
  EraseAllMemory,
  SetWR_Flash,
  DELAY_N100US, 235,
  SET_MAX_COUNT8, 50,
  WaitProgramOperation,		//	<---+
  REGOUT_L,			//	    |
  LOOP_MAX_WaitPgm, 2,		//	->--+
  DELAY_N100US, 40,
  0
};

static const byte SQ_FLashComp_ProgBuf[] = {
  SETADR24,			//Adr   +24bit
  SETCOUNT8,			//Count +8bit  1..200
  ExitReset,

  Init_FA_TBLPAG,
  Skip_FF_2x24Bit, 14,		//	<-------+
  Load_2x24Bit,			//		|
  FLASH_Prog2wr,		//		|
  InitAdr_2xReg_W3W4,		//		|
  W3W4_TO_NVMADR,		//		|
  FLASH_ProgOpen,		//		|
  SetWR_Flash,			//		|
  SET_MAX_COUNT8, 100,		//		|
  DELAY_N10US, 7,		//		|
  WaitProgramOperation,		//	<---+	|
  REGOUT_L,			//	    |	|
  LOOP_MAX_WaitPgm, 2,		//	->--+	|
  LOOP_COUNT8, 16,		//	->------+

  SETADR24,			//Adr   +24bit
  SETCOUNT8,			//Count +8bit  1..200
  InitTBLPAG_W6,
  VISI_W7,			//	
  ReadDataL_word,		//	<---+
  REGOUT_L,			//	    |
  ReadDataH_word,		//	    |
  REGOUT_H,			//	    |
  REGOUT24_COMP,		//	    |
  LOOP_COUNT8, 5,		//	->--+
  0
};

static const byte SQ_FLash_ProgBuf[] = {
  SETADR24,			//Adr   +24bit
  SETCOUNT8,			//Count +8bit  1..200
  ExitReset,

  Init_FA_TBLPAG,
  Skip_FF_2x24Bit, 14,		//	<-------+
  Load_2x24Bit,			//		|
  FLASH_Prog2wr,		//		|
  InitAdr_2xReg_W3W4,		//		|
  W3W4_TO_NVMADR,		//		|
  FLASH_ProgOpen,		//		|
  SetWR_Flash,			//		|
  SET_MAX_COUNT8, 100,		//		|
  DELAY_N10US, 7,		//		|
  WaitProgramOperation,		//	<---+	|
  REGOUT_L,			//	    |	|
  LOOP_MAX_WaitPgm, 2,		//	->--+	|
  LOOP_COUNT8, 16,		//	->------+
  0
};

/* static const byte DE_Read_nxWord[] = {
  SETCOUNT8,			//Count +8bit  1..200
  VISI_W7,			//	<---+
  ReadData_1word,		//	    |
  REGOUT,			//	    |
  LOOP_COUNT8, 3,		//	->--+
  0
}; */

static int SetExecBef(TCpuPgm_dsPIC33E *self, const byte *bef, dword adr, dword nloop) {
  int result;
  byte *TxData;

	TxData = self->TxDbuf;
	result = 0;
	while (*bef) {
	    *TxData = *bef;
	    TxData++; result++;
	    if (*bef == SETADR24) {
		*TxData = adr;	TxData++; result++;
		*TxData = adr >> 8;	TxData++; result++;
		*TxData = adr >> 16;	TxData++; result++;

	    } else if (*bef == SETCOUNT8) {
		*TxData = nloop; TxData++; result++;

	    } else if (*bef == SET_MAX_COUNT8) {
		bef++;
		*TxData = *bef;
		TxData++; result++;

	    } else if (*bef == LOOP_COUNT8) {
		bef++;
		*TxData = *bef;
		TxData++; result++;

	    } else if (*bef == LOOP_MAX_WaitPgm) {
		bef++;
		*TxData = *bef;
		TxData++; result++;

	    } else if (*bef == DELAY_N10US) {
		bef++;
		*TxData = *bef;
		TxData++; result++;

	    } else if (*bef == DELAY_N100US) {
		bef++;
		*TxData = *bef;
		TxData++; result++;

	    } else if (*bef == Skip_FF_2x24Bit) {
		bef++;
		*TxData = *bef;
		TxData++; result++;
	    }
	    bef++;
	}
	return result;
}


static void DoStatus_Data(TCpuPgm_dsPIC33E *self, int Stat, int StatuFu) {
	if (self->OnStatus) self->OnStatus(self, Stat, StatuFu);
}

static void iError(TCpuPgm_dsPIC33E *self, const char *msg, ...) {
  va_list args;
  bool ok;

	va_start(args, msg);
	if (msg) vsnprintf(self->Errbuf, sizeof(self->Errbuf), msg, args);
	va_end(args);
	if ((self->aflag & EFLAG_EnterICSP) && self->rs232ald) {
	    self->sflag = EFLAG_ExitICSP;
	} else {
	    self->sflag = 0;
	}
	self->AktError = 1;
	ok = false;
	if (self->OnError) ok = self->OnError(self, self->Errbuf);
	if (!ok) appError(EMSG_ERR1, self->Errbuf);
	DoStatus_Data(self, STAT_ERROR, STATFU_NONE);
}

static void chk_IObuf_size(TCpuPgm_dsPIC33E *self) {
	if (self->IObuf_size < self->read_len) {
	    self->IObuf_size = (self->read_len + 0x1fff) & 0xffffe000;
	    self->IObuf = xrealloc(self->IObuf, self->IObuf_size);
	}
	self->IO_size = 0;
}

/* ______________________________________________________________________ */
static bool comp_progmem(TCpuPgm_dsPIC33E *self) {
  int i, cfgi;
  dword *sdw, *ddw, adr, adr_cfg, mask_cfg;
  TSegment *seg;
  const TPicDevice *pic_dev;

	pic_dev = self->PicDevice;
	if (!pic_dev) {
	    iError(self, "comp_progmem no PicDevice ???");
	    return false;
	}

	seg = self->HexIOSeg;
	i = seg->end - seg->anf;
	if (self->IO_size != i) {
	    iError(self, "comp_progmem IO_size=%d <> seg_size=%d", 
			self->IO_size, i);
	    return false;
	}

	if (!(self->evflag & EFLAG_COMP_PROGMEM)) {
	    self->evflag |= EFLAG_COMP_PROGMEM;
	    DoStatus_Data(self, STAT_COMPARE, STATFU_START);
	}

	adr_cfg = pic_dev->MaxCfg << 1;
	sdw = (dword*)seg->data;
	adr = self->read_start_adr;
	ddw = (dword*)self->IObuf;
	cfgi = 0;
	i /= 4;
	if (i < 1) return false;
	for (;i>0;i--) {
	    if ((adr >= adr_cfg) && (cfgi < pic_dev->ConfReg[0])) {
		cfgi++;
		mask_cfg = pic_dev->ConfReg[cfgi] | 0x0000ff00;
		if ((*sdw & mask_cfg) != (*ddw & mask_cfg)) {
		    iError(self, "comp_progcfg adr=0x%06x  src=0x%06x <> dest=0x%06x", 
				adr / 2, *sdw & mask_cfg, *ddw & mask_cfg);
		    return false;
		}

	    } else {
		if ((*sdw & 0x00ffffff) != (*ddw & 0x00ffffff)) {
		    iError(self, "comp_progmem adr=0x%06x  src=0x%06x <> dest=0x%06x", 
				adr / 2, *sdw, *ddw);
		    return false;
		}
	    }
	    sdw++; ddw++; adr += 4;
	}
	DoStatus_Data(self, STAT_COMPARE, STATFU_RUN);
	return true;
}

static bool setup_comp_mem(TCpuPgm_dsPIC33E *self) {
  TSegment *seg;

	seg = self->HexIOSeg;
	if (!seg) return false;

	self->read_len = seg->end - seg->anf;
	self->read_start_adr = seg->anf;
	self->read_adr = seg->anf >> 1;
	if (seg->anf < self->HexFileIO->hex_align_end) {
// printf("%s(%d): 0x%06x %d\n", FL, self->read_adr, self->read_len);
	    self->sflag |= EFLAG_READ_N_PROGMEM;
	    self->aflag &= ~EFLAG_READ_N_PROGMEM;
	} else {
// printf("%s(%d): \n", FL);
	    self->sflag |= EFLAG_READ_N_WORD;  /* Read CFG */
	    self->aflag &= ~EFLAG_READ_N_WORD;
	}
	chk_IObuf_size(self);
	return true;
}

static bool chk_flash_cfg(TCpuPgm_dsPIC33E *self) {
	return (self->write_adr & 0xffffff80) == (self->PicDevice->MaxCfg & 0xffffff80);
}

static bool setup_flash_mem(TCpuPgm_dsPIC33E *self) {
  TSegment *seg;

	seg = self->HexIOSeg;
	if (!seg) return false;

	if (self->HexIOSegOffset >= (seg->end - seg->anf)) {
	    seg = seg->next;
	    self->HexIOSeg = seg;
	    if (!seg) return false;
	    self->HexIOSegOffset = 0;
	}

	self->write_len = 256;
	self->write_adr = (seg->anf + self->HexIOSegOffset) >> 1;

	self->sflag |= EFLAG_WRITE_N_PROGMEM;
	self->aflag &= ~EFLAG_WRITE_N_PROGMEM;
	return true;
}

/* ______________________________________________________________________ */
static bool cp_NextTxDData(TCpuPgm_dsPIC33E *self) {
  int send_bef, send_len, i;
  const TICSP_bef *icbef;
  byte *db;
  const byte *cdb;

	self->txflag = 0;

// printf("%s(%d): s=0x%04x a=0x%04x\n", FL, self->sflag, self->aflag);

	if ((self->sflag & EFLAG_PROGRAMMER_ON) && !(self->aflag & EFLAG_PROGRAMMER_ON)) {
 printf("%s(%d): Tx PROGRAMMER_ON\n", FL);
	    self->txflag = EFLAG_PROGRAMMER_ON;
	    self->TxDbuf[0] = 0x00;
	    self->TxDbuf[1] = 0xf3;
	    send_bef = RSBEF_BRIDGE_MODE;
	    send_len = 2;

	} else if ((self->sflag & EFLAG_SETCPUID) && !(self->aflag & EFLAG_SETCPUID)) {
 printf("%s(%d): Tx SETCPUID\n", FL);
	    self->txflag = EFLAG_SETCPUID;
	    self->TxDbuf[0] = 1; //ds33E
	    send_bef = ds33BEF_SETCPUID;
	    send_len = 1;

	} else if ((self->sflag & EFLAG_EnterICSP) && !(self->aflag & EFLAG_EnterICSP)) {
 printf("%s(%d): Tx EnterICSP\n", FL);
	    self->txflag = EFLAG_EnterICSP;
	    self->TxDbuf[0] = 1;
	    send_bef = ds33BEF_ICSP;
	    send_len = 1;

	} else if ((self->sflag & EFLAG_ClrICSP_BEF) && !(self->aflag & EFLAG_ClrICSP_BEF)) {
 printf("%s(%d): Tx ClrICSP_BEF\n", FL);
	    self->ICSP_bef_idx = 0;
	    self->txflag = EFLAG_ClrICSP_BEF;
	    self->TxDbuf[0] = 0;
	    send_bef = ds33BEF_SendICSP_BEF;
	    send_len = 1;

	} else if ((self->sflag & EFLAG_SendICSP_BEF) && !(self->aflag & EFLAG_SendICSP_BEF)) {
 if (!self->ICSP_bef_idx) printf("%s(%d): Tx SendICSP_BEF\n", FL);
	    self->txflag = EFLAG_SendICSP_BEF;
	    icbef = ICSP_bef + self->ICSP_bef_idx;
	    if (icbef->count && icbef->data) {
		self->TxDbuf[0] = 2;
		self->TxDbuf[1] = icbef->count;
		pnmove(self->TxDbuf + 2, icbef->data, icbef->count * 4);
		send_bef = ds33BEF_SendICSP_BEF;
		send_len = (icbef->count * 4) + 2;

	    } else {
		self->TxDbuf[0] = 1;
		send_bef = ds33BEF_SendICSP_BEF;
		send_len = 1;
	    }

	} else if ((self->sflag & EFLAG_ReadDevID) && !(self->aflag & EFLAG_ReadDevID)) {
	    send_len = SetExecBef(self, SQ_Read_nWord, 0xff0000, 1);
 printf("%s(%d): Tx ReadDevID\n", FL);
	    self->txflag = EFLAG_ReadDevID;
	    send_bef = ds33BEF_ExecICSP_BEF;

	} else if ((self->sflag & EFLAG_ClrFLASH) && !(self->aflag & EFLAG_ClrFLASH)) {
	    DoStatus_Data(self, STAT_CLR_FLASH, STATFU_START);
	    send_len = SetExecBef(self, SQ_EraseAllMemory, 0, 50);
 printf("%s(%d): Tx ReadDevID beflen=%d adr=0x%06x len=%d\n", FL, send_len, self->read_adr, self->ReadBlockLen);
	    self->txflag = EFLAG_ClrFLASH;
	    send_bef = ds33BEF_ExecICSP_BEF;

	} else if ((self->sflag & EFLAG_READ_N_WORD) && !(self->aflag & EFLAG_READ_N_WORD)) {
	    self->ReadBlockLen = self->read_len;
	    if (self->ReadBlockLen > 252) self->ReadBlockLen = 252;
	    send_len = SetExecBef(self, SQ_Read_nWord, self->read_adr, (self->ReadBlockLen >> 1) - 1);
// if (!self->IO_size) printf("%s(%d): Tx READ_N_WORD beflen=%d adr=0x%06x len=%d\n", FL, send_len, self->read_adr, self->read_len);
	    self->txflag = EFLAG_READ_N_WORD;
	    send_bef = ds33BEF_ExecICSP_BEF;

	} else if ((self->sflag & EFLAG_READ_N_PROGMEM) && !(self->aflag & EFLAG_READ_N_PROGMEM)) {
	    if (!(self->evflag & EFLAG_READ_N_PROGMEM)) {
		self->evflag |= EFLAG_READ_N_PROGMEM;
		i = STAT_READ;
		if (self->sflag & EFLAG_COMP_PROGMEM) i = STAT_COMPARE;
		DoStatus_Data(self, i, STATFU_START);
	    }

	    self->ReadBlockLen = self->read_len;
	    if (self->ReadBlockLen > 336) self->ReadBlockLen = 336; //336*3/4=252
	    send_len = SetExecBef(self, SQ_Read_nProgMem, self->read_adr, (self->ReadBlockLen >> 2) - 1);
// if (!self->IO_size) printf("%s(%d): Tx READ_N_PROGMEM beflen=%d adr=0x%06x len=%d\n", FL, send_len, self->read_adr, self->read_len);
	    self->txflag = EFLAG_READ_N_PROGMEM;
	    send_bef = ds33BEF_ExecICSP_BEF;

	} else if ((self->sflag & EFLAG_WRITE_N_PROGMEM) && !(self->aflag & EFLAG_WRITE_N_PROGMEM)) {
// if (!self->HexIOSegOffset) printf("%s(%d): Tx WRITE_N_PROGMEM adr=0x%06x len=%d off=%d\n", FL, 
//		self->write_adr, self->write_len, self->HexIOSegOffset);

	    if (!(self->evflag & EFLAG_WRITE_N_PROGMEM)) {
		self->evflag |= EFLAG_WRITE_N_PROGMEM;
		DoStatus_Data(self, STAT_PROGRAMING, STATFU_START);
	    }

	    db = self->TxDbuf;
	    cdb = self->HexIOSeg->data + self->HexIOSegOffset;
	    for (i=(self->write_len>>2);i>0;i--) {
		*db = *cdb; db++; cdb++;
		*db = *cdb; db++; cdb++;
		*db = *cdb; db++; cdb++;
		cdb++;
	    }
	    send_len = (self->write_len>>2) * 3;
	    self->txflag = EFLAG_WRITE_N_PROGMEM;
	    send_bef = ds33BEF_SEND_DATA;

	} else if ((self->sflag & EFLAG_FLASH_PROGMEM) && !(self->aflag & EFLAG_FLASH_PROGMEM)) {
	    if (chk_flash_cfg(self)) {
		send_len = SetExecBef(self, SQ_FLash_ProgBuf, self->write_adr, (self->write_len >> 3) - 1);
	    } else {
		send_len = SetExecBef(self, SQ_FLashComp_ProgBuf, self->write_adr, (self->write_len >> 3) - 1);
	    }
// if (!self->HexIOSegOffset) printf("%s(%d): Tx FLASH_PROGMEM beflen=%d adr=0x%06x len=%d\n", FL, send_len, self->write_adr, self->write_len);
	    self->txflag = EFLAG_FLASH_PROGMEM;
	    send_bef = ds33BEF_ExecICSP_BEF;

	} else if (self->sflag & EFLAG_ExitICSP) {
 printf("%s(%d): Tx ExitICSP\n", FL);
	    self->sflag = EFLAG_ExitICSP|EFLAG_PROGRAMMER_OFF;
	    self->txflag = EFLAG_ExitICSP;
	    self->TxDbuf[0] = 0;
	    send_bef = ds33BEF_ICSP;
	    send_len = 1;

	} else if (self->sflag & EFLAG_PROGRAMMER_OFF) {
 printf("%s(%d): Tx PROGRAMMER_OFF\n", FL);
	    self->sflag = EFLAG_PROGRAMMER_OFF;
	    self->txflag = EFLAG_PROGRAMMER_OFF;
	    self->TxDbuf[0] = 0x00;
	    self->TxDbuf[1] = 0xf2;
	    send_bef = RSBEF_BRIDGE_MODE;
	    send_len = 2;
    
	} else {
	    return false;
	}

	if (!rs232ald_SendData(self->rs232ald, send_bef, self->TxDbuf, send_len)) {
	    self->txflag = 0;
	    iError(self, "SendData txflag=0x%04x", self->txflag);
	    return false;
	}
	return true;
}

/* ______________________________________________________________________ */
static const char *cStrOK = "OK";
static const char *cStrERROR = "ERROR";

static bool cp_RxDData(TCpuPgm_dsPIC33E *self, dword bef, const byte *data, size_t len) {
  int i, stm;
  const TICSP_bef *icbef;
  const word *pw;
  byte *db;
  const byte *sb;
  const char *cStr;
  char *pc;
  char buf[100];

	if (!self->txflag) return false;

	cStr = cStrOK;
	if (bef == RSBEF_ERROR) {
	    cStr = cStrERROR;
	}


	if (self->txflag & EFLAG_PROGRAMMER_ON) {
// printf("%s(%d): Rx PROGRAMMER_ON %s\n", FL, cStr);
	    if (bef == RSBEF_OK) {
		self->aflag |= EFLAG_PROGRAMMER_ON;
	    } else {
		iError(self, "PROGRAMMER_ON \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
		if (self->OnClose) self->OnClose(self, self->AktError);
	    }

	} else if (self->txflag & EFLAG_SETCPUID) {
// printf("%s(%d): Rx SETCPUID %s\n", FL, cStr);
	    if (bef == RSBEF_OK) {
		self->aflag |= EFLAG_SETCPUID;
	    } else {
		iError(self, "SETCPUID \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
		if (self->OnClose) self->OnClose(self, self->AktError);
	    }

	} else if (self->txflag & EFLAG_EnterICSP) {
// printf("%s(%d): Rx EnterICSP %s\n", FL, cStr);
	    if (bef == RSBEF_OK) {
		self->aflag |= EFLAG_EnterICSP;
		self->ICSP_ms = GetTime_ms();
	    } else {
		self->aflag |= EFLAG_EnterICSP;
		iError(self, "EnterICSP \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_ClrICSP_BEF) {
// printf("%s(%d): Rx ClrICSP_BEF %s\n", FL, cStr);
	    if (bef == RSBEF_OK) {
		self->aflag |= EFLAG_ClrICSP_BEF;
	    } else {
		iError(self, "ClrICSP_BEF \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_SendICSP_BEF) {
	    if (bef == RSBEF_OK) {
		icbef = ICSP_bef + self->ICSP_bef_idx;
		if (icbef->count && icbef->data) {
		    self->ICSP_bef_idx++;
		} else {
//    printf("%s(%d): Rx SendICSP_BEF %s Count=%d t=%dms\n", FL, cStr, self->ICSP_bef_idx, GetTime_ms() - self->ICSP_ms);
		    self->aflag |= EFLAG_SendICSP_BEF;
		}
	    } else {
		iError(self, "SendICSP_BEF \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_ReadDevID) {
 printf("%s(%d): Rx ReadDevID %s 0x%02x%02x 0x%02x%02x\n", FL, cStr, data[1], data[0], data[3], data[2]);
	    if ((bef == ds33BEF_ExecICSP_BEF) && (len == 4)) {
		pw = (const word*)data;
		self->PicDevice = GetDevice_Rev(pw, FL_33E, &i);
		if (self->PicDevice) {
		    self->aflag |= EFLAG_ReadDevID;
		    if (self->OnDeviceID) {
			pc = buf;
			ppcmove(&pc, self->PicDevice->name);
			ppcmove(&pc, "  Rev.:");
			if (i >= 0) {
			    inttoppc(&pc, self->PicDevice->rev[i].nr);
			} else {
			    ppcmove(&pc, "???");
			}
			ppcmove(&pc, "    ");
			hexto0xppc(&pc, pw[0], 4);
			*pc = ':'; pc++;
			hexto0xppc(&pc, pw[1], 4);
			if (!(self->PicDevice->Flags & FL_TEST_OK)) {
			    ppcmove(&pc, "    NOT TESTED !!!");
			}

			if (self->OnDeviceID) self->OnDeviceID(self, buf, 
							self->PicDevice->name);
		    }
		} else {
		    if (self->OnDeviceID) self->OnDeviceID(self, "ERROR", NULL);
		    iError(self, "\nDevice ID:   0x%04x\nDevice REV:  0x%04x\nnot found", 
				pw[0], pw[1]);
		}

	    } else {
		if (self->OnDeviceID) self->OnDeviceID(self, "ERROR", NULL);
		iError(self, "ReadDevID \"%s\"", Trs232ald_GetErrorStr(
			 ds33BEF_ExecICSP_BEF, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_ClrFLASH) {
	    DoStatus_Data(self, STAT_CLR_FLASH, STATFU_END);
	    if (bef == ds33BEF_ExecICSP_BEF) {
		self->aflag |= EFLAG_ClrFLASH;
	    } else if ((bef == RSBEF_ERROR) && 
		       (len == 1) && (data[0] == ds33BEF_ExecICSP_BEF)) {
		iError(self, "ClrFLASH TimeOut");
	    } else {
		iError(self, "ClrFLASH \"%s\"", Trs232ald_GetErrorStr(
			 ds33BEF_ExecICSP_BEF, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_READ_N_WORD) {
// printf("%s(%d): 0x%02x  lg=%d  0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", FL, bef, len, data[0], data[1], data[2], data[3], data[4], data[5]);
	    if (self->ReadBlockLen == len) {
		self->ErrorCount = 0;
		pnmove(self->IObuf + self->IO_size, data, len);
		self->read_adr += self->ReadBlockLen;
		self->read_len -= self->ReadBlockLen;
		self->IO_size += self->ReadBlockLen;
		if (self->read_len <= 0) {
		    self->aflag |= EFLAG_READ_N_WORD;
 printf("%s(%d): Rx READ_N_WORD %s len=%d\n", FL, cStr, self->IO_size);
		    if (self->OnDataRead) {
			self->OnDataRead(self, self->IObuf, 
				self->read_start_adr, self->IO_size);
		    }
		}

	    } else {
 printf("%s(%d): ERROR: ReadData adr=0x%06x len=%d <> ret_len=%d\n", FL, self->read_adr, self->ReadBlockLen, (int)len);
		self->ErrorCount++;
		if (self->ErrorCount > 3) {
		    iError(self, "ReadData adr=0x%06x len=%d <> ret_len=%d", 
			    self->read_adr, self->ReadBlockLen, (int)len);
		    self->aflag |= EFLAG_READ_N_WORD;
		}
	    }


	} else if (self->txflag & EFLAG_READ_N_PROGMEM) {
// printf("%s(%d): Rx READ_N_WORD %s\n", FL, cStr);
// printf("%s(%d): 0x%02x  lg=%d  0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", FL, bef, len, data[0], data[1], data[2], data[3], data[4], data[5]);
	    self->NowDataSize += self->ReadBlockLen;
	    if (self->sflag & EFLAG_COMP_PROGMEM) {
		DoStatus_Data(self, STAT_COMPARE, STATFU_RUN);
	    } else {
		DoStatus_Data(self, STAT_READ, STATFU_RUN);
	    }
	    if ((bef == ds33BEF_ExecICSP_BEF) && self->ReadBlockLen == ((len  << 2) / 3)) {
		self->ErrorCount = 0;
//		pnmove(self->IObuf + self->IO_size, data, len);
		sb = data;
		db = self->IObuf + self->IO_size;
		for (i=(self->ReadBlockLen >> 2); i>0;i--) {
		    *db = *sb; db++; sb++;
		    *db = *sb; db++; sb++;
		    *db = *sb; db++; sb++;
		    *db = 0; db++;
		}
		self->read_adr += self->ReadBlockLen >> 1;
		self->read_len -= self->ReadBlockLen;
		self->IO_size += self->ReadBlockLen;
		if (self->read_len <= 0) {
		    self->aflag |= EFLAG_READ_N_PROGMEM;
// printf("%s(%d): Rx READ_N_PROGMEM %s len=%d\n", FL, cStr, self->IO_size);
		    if (self->OnDataRead) {
			self->OnDataRead(self, self->IObuf, 
				self->read_start_adr, self->IO_size);
		    }
		    if (self->sflag & EFLAG_COMP_PROGMEM) {
			stm = STATFU_END;
			if (comp_progmem(self)) {
// printf("%s(%d): Rx COMP_PROGMEM %s\n", FL, cStr);
			    self->HexIOSeg = self->HexIOSeg->next;
			    if (!setup_comp_mem(self)) self->HexIOSeg = NULL;
			} else {
			    stm = STATFU_END | STATFU_ERROR;
			    self->HexIOSeg = NULL;
			}

			if (!self->HexIOSeg) {
			    self->sflag &= ~EFLAG_COMP_PROGMEM;
			    DoStatus_Data(self, STAT_COMPARE, stm);
			}
		    } else {
			DoStatus_Data(self, STAT_READ, STATFU_END);
		    }
		}

	    } else {
 printf("%s(%d): ERROR: ReadData adr=0x%06x len=%d <> ret_len=%d\n", FL, self->read_adr, self->ReadBlockLen, (int)len);
		iError(self, "READ_N_PROGMEM \"%s\"", Trs232ald_GetErrorStr(
			ds33BEF_ExecICSP_BEF, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_WRITE_N_PROGMEM) {
// if (!self->HexIOSegOffset) printf("%s(%d): Rx WRITE_N_PROGMEM %s\n", FL, cStr);

	    if (bef == RSBEF_OK) {
		self->aflag |= EFLAG_WRITE_N_PROGMEM;
	    } else {
		iError(self, "WRITE_N_PROGMEM \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
	    }



	} else if (self->txflag & EFLAG_FLASH_PROGMEM) {
// if (!self->HexIOSegOffset) printf("%s(%d): Rx FLASH_PROGMEM %s\n", FL, cStr);

	    self->NowDataSize += self->write_len;
	    if ((bef == ds33BEF_ExecICSP_BEF) && !len) {
		DoStatus_Data(self, STAT_PROGRAMING, STATFU_RUN);
		self->HexIOSegOffset += self->write_len;
		if (chk_flash_cfg(self)) {
		    DoStatus_Data(self, STAT_PROGRAMING, STATFU_END);
		    self->aflag |= EFLAG_FLASH_PROGMEM;
		    if (setup_comp_mem(self)) {
			self->sflag |= EFLAG_COMP_PROGMEM;  /* Read CFG */
		    }

		} else if (!setup_flash_mem(self)) {
		    DoStatus_Data(self, STAT_PROGRAMING, STATFU_END);
		    self->aflag |= EFLAG_FLASH_PROGMEM;
		}

	    } else if ((bef == ds33BEF_ExecICSP_BEF) && (len == 7)) {
// printf("%s(%d): 0x%02x  lg=%d  0x%06x  0x%06x <> 0x%06x\n", FL, bef, len, (data[0] * 2 / 3) + self->write_adr, data[1] + (data[2] << 8) + (data[3] << 16), data[4] + (data[5] << 8) + (data[6] << 16));
		DoStatus_Data(self, STAT_PROGRAMING, STATFU_RUN|STATFU_ERROR);
		iError(self, "Flash Data adr=0x%06x src=0x%06x <> dest=0x%06x", 
			(data[0] * 2 / 3) + self->write_adr, 
			data[1] + (data[2] << 8) + (data[3] << 16), 
			data[4] + (data[5] << 8) + (data[6] << 16));
		self->aflag |= EFLAG_FLASH_PROGMEM;
	    } else {
		DoStatus_Data(self, STAT_PROGRAMING, STATFU_RUN|STATFU_ERROR);
		iError(self, "FLASH_PROGMEM \"%s\"", Trs232ald_GetErrorStr(
			 RSBEF_OK, bef, data, len, true));
	    }

	} else if (self->txflag & EFLAG_ExitICSP) {
	    if (self->BridgeModus) {
// printf("%s(%d): Rx ExitICSP %s\n", FL, cStr);
		self->sflag = EFLAG_PROGRAMMER_OFF;
	    } else {
 printf("%s(%d): Rx ExitICSP %s t=%dms\n", FL, cStr, GetTime_ms() - self->Start_ms);
		self->sflag = EFLAG_PROGRAMMER_OFF;
		self->aflag = 0;
		self->txflag = 0;
		self->ICSP_bef_idx = 0;
		self->rs232ald = NULL;
		if (self->OnClose) self->OnClose(self, self->AktError);
		DoStatus_Data(self, STAT_CLOSE, STATFU_NONE);
	    }

	} else if (self->txflag & EFLAG_PROGRAMMER_OFF) {
 printf("%s(%d): Rx PROGRAMMER_OFF %s t=%dms\n", FL, cStr, GetTime_ms() - self->Start_ms);
	    self->sflag = 0;
	    self->aflag = 0;
	    self->txflag = 0;
	    self->ICSP_bef_idx = 0;
	    self->rs232ald = NULL;
	    if (self->OnClose) self->OnClose(self, self->AktError);
	    DoStatus_Data(self, STAT_CLOSE, STATFU_NONE);

	}
	return true;
}


static bool cp_open(TCpuPgm_dsPIC33E *self, Trs232ald *rs232ald) {
	self->Start_ms = GetTime_ms();
	self->AktError = 0;
	self->rs232ald = rs232ald;
	self->sflag = FLAGS_OPEN;
	self->aflag = 0;
	self->evflag = 0;
	self->GesDataSize = 0;
	self->NowDataSize = 0;
	DoStatus_Data(self, STAT_OPEN, STATFU_NONE);
	return true;
}

static bool cp_close(TCpuPgm_dsPIC33E *self, bool force) {
	if (force) {
	    if (!self->rs232ald) return true;
	    self->sflag = EFLAG_SETCPUID|EFLAG_ExitICSP;
	} else {
	    self->sflag |= EFLAG_SETCPUID|EFLAG_ExitICSP;
	}
	return true;
}

static bool cp_read(TCpuPgm_dsPIC33E *self, dword adr, size_t len) {

	if (self->AktError) return false;

	if (adr & 1) {
	    iError(self, "ReadData Adr. not align 2 0x%06x", adr);
	    return false;
	}
	if (adr >= 0x01000000) {
	    iError(self, "ReadData Adr. out of range 0x%08x", adr);
	    return false;
	}
	if (len & 1) {
	    iError(self, "ReadData len. not align 2 %d", len);
	    return false;
	}
	if ((len >= 0x01000000) || ((adr + len) > 0x01000000)) {
	    iError(self, "ReadData Adr.+len out of range 0x%08x+0x%08x+0x%08x", adr, len, adr+len);
	    return false;
	}

	self->read_len = len;
	self->read_start_adr = adr;
	self->read_adr = adr;
	self->sflag = FLAGS_OPEN|EFLAG_READ_N_WORD;
	self->aflag &= ~EFLAG_READ_N_WORD;
	if (self->IObuf_size < len) {
	    self->IObuf_size = (len + 0x1fff) & 0xffffe000;
	    self->IObuf = xrealloc(self->IObuf, self->IObuf_size);
	}
	self->IO_size = 0;
	return true;
}

static bool cp_read_prog_mem(TCpuPgm_dsPIC33E *self, dword adr, size_t len) {

	if (self->AktError) return false;

	if (adr & 3) {
	    iError(self, "ReadData Adr. not align 4 0x%06x", adr);
	    return false;
	}
	if (adr >= 0x00100000) {
	    iError(self, "ReadData Adr. out of range 0x%08x", adr);
	    return false;
	}
	if (len & 3) {
	    iError(self, "ReadData len. not align 4 %d", len);
	    return false;
	}
	if ((len >= 0x00100000) || ((adr + len) > 0x00100000)) {
	    iError(self, "ReadData Adr.+len out of range 0x%08x+0x%08x+0x%08x", adr, len, adr+len);
	    return false;
	}

	self->read_len = len; //(len >> 1) * 3;
	self->read_start_adr = adr;
	self->read_adr = adr >> 1;
	self->sflag = FLAGS_OPEN|EFLAG_READ_N_PROGMEM;
	self->aflag &= ~EFLAG_READ_N_PROGMEM;
	chk_IObuf_size(self);
	return true;
}


static void SetDataSize(TCpuPgm_dsPIC33E *self) {
  TSegment *seg;

	self->GesDataSize = 0;
	self->NowDataSize = 0;
	seg = self->HexIOSeg;
	while (seg) {
	    self->GesDataSize += seg->end - seg->anf;
	    seg = seg->next;
	}
}

static bool cp_comp_mem(TCpuPgm_dsPIC33E *self) {

	if (self->AktError) return false;
	self->sflag = FLAGS_OPEN|EFLAG_COMP_PROGMEM;
	self->aflag &= ~(EFLAG_READ_N_WORD|EFLAG_READ_N_PROGMEM|EFLAG_COMP_PROGMEM);
	self->HexIOSeg = self->HexFileIO->Seg0;
	SetDataSize(self);
	return setup_comp_mem(self);
}

static bool cp_flash_mem(TCpuPgm_dsPIC33E *self) {

	if (self->AktError) return false;
  /* EFLAG_FLASH_PROGMEM + Read CFG and Compare */
	self->sflag = FLAGS_OPEN|EFLAG_ClrFLASH|EFLAG_FLASH_PROGMEM;
	self->aflag &= ~(EFLAG_READ_N_WORD|EFLAG_READ_N_PROGMEM|EFLAG_ClrFLASH|EFLAG_FLASH_PROGMEM);
	self->HexIOSeg = self->HexFileIO->Seg0;
	self->HexIOSegOffset = 0;
	SetDataSize(self);
	return setup_flash_mem(self);
}

static bool cp_cpureset(TCpuPgm_dsPIC33E *self, Trs232ald *rs232ald, bool force) {
	if (!self->rs232ald) self->Start_ms = GetTime_ms();
	self->rs232ald = rs232ald;
	return cp_close(self, force);
}

/*_____________________________ Destroy _________________________________*/
/* static bool TCpuPgm_dsPIC33E__Destroy(TCpuPgm_dsPIC33E *self) {
	return TCpuPgm__Destroy((TCpuPgm*)self);
} */

/*_____________________________ Create __________________________________*/
void *TCpuPgm_dsPIC33E_Create() {
  TCpuPgm_dsPIC33E *self;


	self = TCpuPgm_Create_struct_init(sizeof(TCpuPgm_dsPIC33E), wtyp_Trs232);
	if (!self) return NULL;
	self->dsFL_33 = FL_33E;
//	self->destroy		= (TDESTROY)TCpuPgm_dsPIC33E__Destroy;
	self->open		= cp_open;
	self->close		= cp_close;
	self->read		= cp_read;
	self->read_prog_mem	= cp_read_prog_mem;
	self->comp_mem		= cp_comp_mem;
	self->flash_mem		= cp_flash_mem;
	self->cpureset		= cp_cpureset;
	self->RxDData		= cp_RxDData;
	self->NextTxDData	= cp_NextTxDData;
	self->RxDbuf = xmalloc(256);
	self->TxDbuf = xmalloc(256);
	self->IObuf_size = 0x2000;
	self->IObuf = xmalloc(self->IObuf_size);
	self->HexFileIO->hex_align = 256;
	self->HexFileIO->hex_align_anf = 0;
	self->HexFileIO->hex_align_end = 0x0007ffff;
	return self;
}
