/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "PIC33pgm.h"
#include "tpixmap.h"

#define TCSelf TPnlcrt
typedef struct TPnlcrt {
  AlgTWins
  TWins *pnlTop;
  TWins *pnlcrt;
  TWins *pnlcrt_graf;
  TChkBox *cb2x;
  TPixMap *pxm;
  TWins *lblcrt_none;
  TWins *pnlBottom;
  TChkBox *cbRDrep;
  TButton *buRD;
  dword CRT;
  dword CRT_Color;
  dword CRT_Width;
  dword CRT_Height;
  dword CRT_mode;
  int rs_byte;
  int rs_line_byte;
  int rs_line_seg;
  int rs_line_seg_byte;
  int rs_line_idx;
  int border_width;
} TPnlcrt;
#undef TCSelf

#define CMODE_2X	1

static TPnlcrt *pnlcrt;
static int DoCRT;
#define DoCRT_RD 0x01

static void crt_chk_size(TPnlcrt *self) {
  int w, h;

	w = self->CRT_Width;
	h = self->CRT_Height;

	if (self->cb2x->Checked && (w < 320) && (h < 200)) {
	    w *= 2;
	    h *= 2;
	    self->CRT_mode |= CMODE_2X;
	} else {
	    self->CRT_mode &= ~CMODE_2X;
	}

	w += self->border_width * 2;
	h += self->border_width * 2;

	if (!self->pxm || (w != self->pxm->width) || (h != self->pxm->width)) {
	    Tfree(self->pxm);
	    self->pxm = TPixMap_Create_depth(w, h, 0);
	    TPixMap_CreateXimg(self->pxm);
	    SetWinSize(self->pnlcrt_graf, w + 2, h + 2);
	    TImage_ClsRGB((TImage*)self->pxm, 0xe0e0ff);
	}
}

static void crt_mono(TPnlcrt *self, const byte *data, size_t lg) {
  int x, y, xm, ym, h;
  dword v, col;
  const byte *dl, *d;

	if (self->CRT_mode & CMODE_2X) {
	    h = lg / self->rs_line_byte;
	    dl = data;
	    ym = (self->rs_line_idx * 2) + self->border_width;
	    for (y=h;y>0;y--) {
		d = dl;
		v = 0;
		for (x=0;x<self->CRT_Width;x++) {
		    if ((x & 0x07) == 0) {
			v = *d; d++;
		    }
		    col = 0xffffff;
		    if (v & 1) col = 0x000000;
		    xm = (x * 2) + self->border_width;
		    TImage_FillRectRGB((TImage*)self->pxm, col, xm, ym, 2, 2);
		    v >>= 1;
		}
		dl += self->rs_line_byte;
		ym += 2;
		self->rs_line_idx++;
	    }

	} else {
	    h = lg / self->rs_line_byte;
	    dl = data;
	    ym = self->rs_line_idx + self->border_width;
	    for (y=h;y>0;y--) {
		d = dl;
		v = 0;
		for (x=0;x<self->CRT_Width;x++) {
		    if ((x & 0x07) == 0) {
			v = *d; d++;
		    }
		    col = 0xffffff;
		    if (v & 1) col = 0x000000;
		    xm = x + self->border_width;
		    TImage_FillRectRGB((TImage*)self->pxm, col, xm, ym, 1, 1);
		    v >>= 1;
		}
		dl += self->rs_line_byte;
		ym ++;
		self->rs_line_idx++;
	    }
	}

}


static void buRD_OnClick(TButton *self, long KeyState, bool DblClick) {
	DoCRT = DoCRT_RD;
	pnlcrt->rs_line_idx = 0;
}

static void pnlcrt_graf_OnPaint(TWins *self) {
	if (!pnlcrt->pxm) return;
	TPixMap_PaintToWin(pnlcrt->pxm, self->WinID, 1, 1);
}

static void pnlBottom_OnResize(TWins *self) {
  int x;

	x = self->width - pnlcrt->buRD->width - 48;
	if (x < 100) x = 100;
	SetLeft((TWins*)pnlcrt->buRD, x);
	x += pnlcrt->buRD->width + 4;
	SetLeft((TWins*)pnlcrt->cbRDrep, x);
}

/*____________________________ TPnlcrt_Alg Create ________________________*/
static void TPnlcrt_CreateParams(TWins_Create_struct *wcs) {
  TPnlcrt *self;

	self = wcs->wins;
	self->subClass = (TWins*)self;
//	self->destroy = (TDESTROY)TPnlcrt_Alg__Destroy;
	self->width = 400;
	self->height = 300;
	self->Align = alClient;
}

static TPnlcrt *TPnlcrt_Create(TWins *Parent) {
  TWins_Create_struct wcs;
  TPnlcrt *self;

	TWins_Create_struct_init(&wcs, sizeof(TPnlcrt), wtyp_TPnlcrt);
	wcs.Parent = Parent;
	wcs.CreateParams = TPnlcrt_CreateParams;
	TWins_DefaultConf(&wcs);

	self = wcs.wins;

	Set_Parent((TWins*)self, Parent);

	self->pnlTop = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, mBorder, alTop);
	self->pnlcrt = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, mBorder, alClient);
	self->pnlBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, mBorder, alBottom);
	self->pnlBottom->OnResize = pnlBottom_OnResize;

	self->cb2x = TChkBox_CreateS((TWins*)self->pnlTop, 2, 2, 38, 17, "2x");


	self->lblcrt_none = TWins_CreateLabel(self->pnlcrt, 2, 2, 100, 40, "CRT NONE", 0, mBorder|mBorderL);
	SetColor(self->lblcrt_none, 0xe0e0e0);
	self->pnlcrt_graf = TWins_CreateLabel(self->pnlcrt, 2, 2, 100, 40, NULL, 0, mBorder|mBorderL);
	SetVisible(self->pnlcrt_graf, false);
	self->pnlcrt_graf->OnPaint = pnlcrt_graf_OnPaint;

	self->buRD = TButton_CreateS((TWins*)self->pnlBottom, 300,  1, 60, 18, "RD");
	self->buRD->OnClick = buRD_OnClick;
	self->cbRDrep = TChkBox_CreateS((TWins*)self->pnlBottom, 270, 2, 38, 17, "rep");

	return self;
}

void init_pg_crt(TPage *self) {
	pnlcrt = TPnlcrt_Create((TWins*)self);
}


void set_pg_crt(TSysInfo0 *SysInfo0) {
  int w;

	pnlcrt->CRT = 0;
	pnlcrt->CRT_mode = 0;
	if (SysInfo0 && SysInfo0->CRT && SysInfo0->CRT_Width && SysInfo0->CRT_Height) {
	    pnlcrt->CRT = SysInfo0->CRT;
	    pnlcrt->CRT_Color = SysInfo0->CRT_Color;
	    pnlcrt->CRT_Width = SysInfo0->CRT_Width;
	    pnlcrt->CRT_Height = SysInfo0->CRT_Height;
	}
//  printf("%s(%d): 0x%02x 0x%02x 0x%04x 0x%04x 0x%04x\n", FL, SysInfo0->HW_Type, SysInfo0->CPU_Factory, SysInfo0->CPU_Type, SysInfo0->CPU_Ver, SysInfo0->DataArt);
//  printf("%s(%d): %d %d %d %d\n", FL, SysInfo0->CRT, SysInfo0->CRT_Color, SysInfo0->CRT_Width, SysInfo0->CRT_Height);

	if (pnlcrt->CRT & BSI0_CRT_GRAFIK) {
	    if (pnlcrt->CRT_Color <= 1) {
		pnlcrt->rs_line_byte = (pnlcrt->CRT_Width + 7) >> 3;
		pnlcrt->rs_byte = pnlcrt->rs_byte * pnlcrt->CRT_Height;
		if (pnlcrt->rs_line_byte < 100) {
		    w = 200 / pnlcrt->rs_line_byte;
		    if (w > pnlcrt->CRT_Height) w = pnlcrt->CRT_Height;
		    pnlcrt->rs_line_seg = w;
		} else {
		    pnlcrt->rs_line_seg = 1;
		}
		pnlcrt->rs_line_seg_byte = pnlcrt->rs_line_seg * pnlcrt->rs_line_byte;
		pnlcrt->border_width = 2;
	    } else {
		goto no_crt;
	    }


	    SetVisible(pnlcrt->lblcrt_none, false);
	    crt_chk_size(pnlcrt);
	    SetVisible(pnlcrt->pnlcrt_graf, true);

	} else {
no_crt:
	    ppTfree(&pnlcrt->pxm);
	    SetVisible(pnlcrt->pnlcrt_graf, false);
	    SetVisible(pnlcrt->lblcrt_none, true);
	}
}

void set_en_pg_crt(TFormMain *self, bool en) {
	DoCRT = 0;
	if (!pnlcrt) return;
	if (pnlcrt->buRD) SetEnabled((TWins*)pnlcrt->buRD, en);
}

bool run_pg_crt(TFormMain *self) {
  byte buf[8];

	if (pnlcrt && (DoCRT || (pnlcrt->cbRDrep->Checked && 
				 (self->pgc->AktPage == self->pg_crt)))) {
	    buf[0] = RSDT_CRT_GRAFIK;
	    buf[1] = 0;
	    buf[2] = 0;
	    buf[3] = 0;
	    buf[4] = pnlcrt->rs_line_seg_byte;
	    rs232ald_SendData(Get_rs232(), RSBEF_READ_DATA_TYPE, buf, 5);
	    DoCRT = 0;
	    crt_chk_size(pnlcrt);
	    return true;
	}
	DoCRT = 0;
	return false;
}

bool OnRxDData_pg_crt(Trs232ald *self, dword bef, const byte *data, size_t lg) {
  int x, h;
  byte buf[8];



	if (bef == RSBEF_ERROR) {
	    rs232ald_ErrPrint(self->LastTxDBef, bef, data, lg);

	} else if (pnlcrt && pnlcrt->pxm) {
// printf("%s(%d): 0x%02x  lg=%d  0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", FL, bef, lg, data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7]);
	    if (pnlcrt->CRT_Color <= 1) {
		crt_mono(pnlcrt, data, lg);
	    }

	    if (pnlcrt->rs_line_idx < pnlcrt->CRT_Height) {
		h = pnlcrt->CRT_Height - pnlcrt->rs_line_idx;
		if (h > pnlcrt->rs_line_seg) h = pnlcrt->rs_line_seg;
		x = pnlcrt->rs_line_idx * pnlcrt->rs_line_byte;

		buf[0] = RSDT_CRT_GRAFIK;
		buf[1] = x;
		buf[2] = x >> 8;
		buf[3] = 0;
		buf[4] = h * pnlcrt->rs_line_byte;
		rs232ald_SendData(Get_rs232(), RSBEF_READ_DATA_TYPE, buf, 5);
		return true;
	    }

	    pnlcrt->rs_line_idx = 0;
	    TPixMap_XPutImage(pnlcrt->pxm);
	    TPixMap_PaintToWin(pnlcrt->pxm, pnlcrt->pnlcrt_graf->WinID, 1, 1);
	}
	return false;
}

bool save_as_pg_crt_ok(TFormMain *self) {
	return pnlcrt->pxm != NULL;
}

void pg_crt_SaveFile(TFormMain *mf) {
  char *s;
  const char *pc;

	if (!pnlcrt || !pnlcrt->pxm) return;

	pc = TpcIni_RD(appDat.appIni, APPNAME, "crt_LastFile");
	if (pc) {
	    s = GetFilePath(pc);
	    if (s) {
		File_Dlg_SetPath(mf->FileSaveLCD_Dlg, s);
		xfree(s);
	    }
	}

	File_Dlg_SetPattern(mf->FileSaveLCD_Dlg, "*.png", NULL);

	if (File_Dlg_Execute(mf->FileSaveLCD_Dlg)) {
	    s = NULL;
	    File_Dlg_GetEdFileName(mf->FileSaveLCD_Dlg, &s);
	    if (!pccmp(GetFileExt_pc(s), "png")) {
		ppcadd(&s, ".png");
	    }

	    if (TImage_Write_PNG((TImage*)pnlcrt->pxm, s, 
			fuWR_NoMask|fuWR_use_pallette, 10) != IMGWR_OK) goto err;
	    TpcIni_WR(appDat.appIni, APPNAME, "crt_LastFile", s);
	    TpcIni_Save(appDat.appIni);
err:
	    xfree(s);
	}
}
