/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "PIC33pgm.h"

#if USE_I2C_BRIDGE

#include "i2c_rs232.h"
#include "thexedit.h"
#include "tfio.h"

#ifndef BSI0EX_BRIDGE_I2C_MASK
#define BSI0EX_BRIDGE_I2C_MASK		0x03
//#define BSI0EX_BRIDGE_I2C_16B		0x01
#define BSI0EX_BRIDGE_I2C_32B		0x02
#define BSI0EX_BRIDGE_I2C_64B		0x03
#endif


#define AlgTPnlI2C AlgTWins \
  THexEdit *HexEdit; \
  TWins *pnlBottom; \
  TChkBox *cbEdit; \
  TButton *buRD; \
  TButton *buWR; \
  bool (*OnRxDData)(Trs232ald *self, dword bef, const byte *data, size_t lg); \
  bool (*NextTxDData)(struct TCSelf *self, Trs232ald *rs232); \
  TI2C_Data i2c_wrData; \
  int i2c_size_count; \
  int i2c_adr; \
  int i2c_adr_count;

//  bool (*SetWR_heat)(struct TCSelf *self);
//  bool (*SetWR_data)(struct TCSelf *self);
//  bool (*SetRD_heat)(struct TCSelf *self);

//  TI2C_RD_Data i2c_rdData;
//  void (*OnReadData)(struct TCSelf *self);
//  void (*OnSendData)(struct TCSelf *self);

#define TCSelf TPnlI2C
typedef struct TPnlI2C {
  AlgTPnlI2C
} TPnlI2C;
#undef TCSelf

#define TCSelf TPnlI2C_Alg
typedef struct TPnlI2C_Alg {
  AlgTPnlI2C
} TPnlI2C_Alg;
#undef TCSelf

#define DoI2C_RD 0x01
#define DoI2C_WR 0x02

static TPnlI2C *pnlI2C;
static int DoI2C;

static void buWR_OnClick(TButton *self, long KeyState, bool DblClick) {
  TPnlI2C *pnl;
//  bool ok;

	pnl = self->subClass;
	Fill0(&pnl->i2c_wrData, sizeof(pnl->i2c_wrData));
/*	ok = false;
	if ((pnl->SetWR_heat) && (pnl->SetWR_data)) {
	    ok = pnl->SetWR_heat(pnl) && pnl->SetWR_data(pnl);
	}
	if (ok) */
	DoI2C = DoI2C_WR;
}

static void buRD_OnClick(TButton *self, long KeyState, bool DblClick) {
  TPnlI2C *pnl;
//  bool ok;

	pnl = self->subClass;
	Fill0(&pnl->i2c_wrData, sizeof(pnl->i2c_wrData));
/*	ok = false;
	if (pnl->SetRD_heat) {
	    ok = pnl->SetRD_heat(pnl);
	}
	if (ok) DoI2C = DoI2C_RD; */
	DoI2C = DoI2C_RD;
}

/*____________________________ TPnlI2C_Alg ev ____________________________*/
static bool edI2C_size_Chk(TFormMain *mf, int *rsize) {
  int b, size;

	mf = appDat.obj;
	b = pctoint(GetCaption_pc((TWins*)mf->cbI2C_adr));

	size = pctoint(GetCaption_pc((TWins*)mf->edI2C_size));
	if (b == 1) {
	    if ((size < 1) || (size > 256)) {
		appError(EMSG_ERR1, "Size=%d use 1..256", size);
		return false;
	    }
	} else if (b == 2) {
	    if ((size < 1) || (size > 65536)) {
		appError(EMSG_ERR1, "Size=%d use 1..65536", size);
		return false;
	    }
	} else {
	    if ((size < 1) || (size > 16)) {
		appError(EMSG_ERR1, "Size=%d use 1..16", size);
		return false;
	    }
	}
	if (rsize) *rsize = size;
	return true;
}

static bool TPnlI2C_Set_heat(TPnlI2C_Alg *self) {
  TFormMain *mf;
  int i, adr;

	mf = appDat.obj;

	i = pctoint(GetCaption_pc((TWins*)mf->cbI2C_id)) << 4;
	i |= pctoint(GetCaption_pc((TWins*)mf->cbI2C_id_adr)) << 1;
	self->i2c_wrData.id = i;

	i = pctoint(GetCaption_pc((TWins*)mf->cbI2C_adr));
	adr = 0;
	if (i) {
	    adr = pctoint(GetCaption_pc((TWins*)mf->edI2C_adr));
	    if (i == 1) {
		if ((adr < 0) || (adr > 0xff)) {
		    appError(EMSG_ERR1, "Adr=%d(0x%02x) use 0..255(0xff)", i, i);
		    return false;
		}
	    } else {
		if ((adr < 0) || (adr > 0xffff)) {
		    appError(EMSG_ERR1, "Adr=%d(0x%04x) use 0..65535(0xffff)", i, i);
		    return false;
		}
		self->i2c_wrData.flag = I2C_bADR16Bit;
	    }

	} else {
	    self->i2c_wrData.flag = I2C_bNoADR;
	}
	self->i2c_adr = adr;
	self->i2c_wrData.adr[0] = adr & 0xff;
	self->i2c_wrData.adr[1] = (adr >> 8) & 0xff;
 printf("%s(%d): ID=0x%02x flag=0x%02x adr=0x%02x%02x\n", FL, self->i2c_wrData.id, self->i2c_wrData.flag, self->i2c_wrData.adr[0], self->i2c_wrData.adr[1]);
	self->i2c_adr_count = 0;
	return true;
}

static int GetMax_i2c_Buf_size() {
	switch (CPU_Info0.Extra & BSI0EX_BRIDGE_I2C_MASK) {
	case BSI0EX_BRIDGE_I2C_32B: return 32;
	case BSI0EX_BRIDGE_I2C_64B: return 64;
	}
	return 16;
}

static bool TPnlI2C_Alg_SetRD_heat(TPnlI2C_Alg *self) {
  TFormMain *mf;
  int size, maxd;

	mf = appDat.obj;
	self->i2c_wrData.cmd = I2C_READ;

	if (!TPnlI2C_Set_heat(self)) return false;

	if (!edI2C_size_Chk(mf, &size)) return false;

	self->i2c_size_count = size;
	self->i2c_wrData.lg = size;
	maxd = GetMax_i2c_Buf_size();
	if (size > maxd) self->i2c_wrData.lg = maxd;

	FIO_SetMemSize(self->HexEdit->fio, size);
	AdjustSize((TWins*)self->HexEdit);
	Refresh((TWins*)self->HexEdit);
	return true;
}

static bool TPnlI2C_Alg_SetWR_data(TPnlI2C_Alg *self) {
  char *data;

	data = FIO_GetDataP(self->HexEdit->fio, self->i2c_adr_count, 16);
	if (!data) return false;
	pnmove(self->i2c_wrData.data, data, self->i2c_wrData.lg);
	return true;
}

static bool TPnlI2C_Alg_SetWR_heat(TPnlI2C_Alg *self) {
  TFormMain *mf;
  int size, maxd;

	mf = appDat.obj;
	self->i2c_wrData.cmd = I2C_WRITE;

	if (!TPnlI2C_Set_heat(self)) return false;

	if (!edI2C_size_Chk(mf, &size)) return false;

	self->i2c_size_count = size;
	self->i2c_wrData.lg = size;
	maxd = GetMax_i2c_Buf_size();
	if (size > maxd) self->i2c_wrData.lg = maxd;

	if (FIO_GetSize(self->HexEdit->fio) < size) {
 printf("%s(%d): ERROR: FIO_GetSize ???\n", FL);
	    return false;
	}

	return TPnlI2C_Alg_SetWR_data(self);
}

static bool TPnlI2C_Alg_NextTxDData(TPnlI2C_Alg *self, Trs232ald *rs232) {
  int bef;

	if (DoI2C == DoI2C_WR) {
	    if (!TPnlI2C_Alg_SetWR_heat(self)) return false;
	    bef = RSBEF_USER;
	    if (CPU_Info0.Extra & BSI0EX_BRIDGE_I2C_MASK) bef = RSBEF_I2C_BRIDGE;
	    rs232ald_SendData(rs232, bef, (const byte*)&self->i2c_wrData, 
					self->i2c_wrData.lg + 6);
	    return true;

	} else if (DoI2C == DoI2C_RD) {
	    if (!TPnlI2C_Alg_SetRD_heat(self)) return false;
	    bef = RSBEF_USER;
	    if (CPU_Info0.Extra & BSI0EX_BRIDGE_I2C_MASK) bef = RSBEF_I2C_BRIDGE;
	    rs232ald_SendData(rs232, bef, (const byte*)&self->i2c_wrData, 6);
	    return true;
	}
	return false;
}

static bool TPnlI2C_Alg_OnRxDData(Trs232ald *self, dword bef, const byte *data, size_t lg) {
  const TI2C_RD_Data *d;
  int i, adr, maxd, wrData_lg;

	if (bef == RSBEF_ERROR) {
	    rs232ald_ErrPrint(self->LastTxDBef, bef, data, lg);

	} else if (pnlI2C) {
	    d = (const TI2C_RD_Data*)data;
	    if (d->cmd == I2C_ERROR) {
		MsgBox_Error((TWins*)pnlI2C, " I2C IO\n\n device not found");
		return false;
	    }

// printf(".size=%d adr=0x%04x  lg=%d -> lg=%d\n",pnlI2C->i2c_size_count,  pnlI2C->i2c_adr_count, pnlI2C->i2c_wrData.lg, lg);
	    if (DoI2C == DoI2C_RD) {
		if ((lg - 2) != pnlI2C->i2c_wrData.lg) {
		    MsgBox_Error((TWins*)pnlI2C, "I2C (RD) read data size %d <> %d", lg - 2, pnlI2C->i2c_wrData.lg);
		    return false;
		}

		FIO_OverWrite(pnlI2C->HexEdit->fio, pnlI2C->i2c_adr_count, 
				    (const char*)d->data, pnlI2C->i2c_wrData.lg);
		Refresh((TWins*)pnlI2C->HexEdit);

	    } else if (DoI2C == DoI2C_WR) {
		if (lg != 2) {
		    MsgBox_Error((TWins*)pnlI2C, "I2C (WR) read data size %d <> 2", pnlI2C->i2c_wrData.lg);
		    return false;
		}

	    } else {
		return false;
	    }
/*
	    pc = buf;
	    for (i=0;i<16;i++) {
		hextoppc(&pc, d->data[i], 2 + kSPACE_ADD);
	    }
 printf(" cmd=0x%02x id=0x%02x  %s\n", d->cmd, d->id, buf);
*/
	    pnlI2C->i2c_size_count -= pnlI2C->i2c_wrData.lg;
	    pnlI2C->i2c_adr_count += pnlI2C->i2c_wrData.lg;
	    if (pnlI2C->i2c_size_count > 0) {
// printf(" size=%d adr=0x%04x  lg=%d\n",pnlI2C->i2c_size_count,  pnlI2C->i2c_adr_count, pnlI2C->i2c_wrData.lg);
		maxd = GetMax_i2c_Buf_size();
		wrData_lg = pnlI2C->i2c_size_count;
		if (wrData_lg > maxd) wrData_lg = maxd; 
		pnlI2C->i2c_wrData.lg = wrData_lg;

		adr = pnlI2C->i2c_adr + pnlI2C->i2c_adr_count;
		pnlI2C->i2c_wrData.adr[0] = adr & 0xff;
		pnlI2C->i2c_wrData.adr[1] = (adr >> 8) & 0xff;
		if (DoI2C == DoI2C_RD) {
		    wrData_lg = 0;
		} else {
		    if (!TPnlI2C_Alg_SetWR_data((TPnlI2C_Alg*)pnlI2C)) return false;
		}

		i = RSBEF_USER;
		if (CPU_Info0.Extra & BSI0EX_BRIDGE_I2C_MASK) i = RSBEF_I2C_BRIDGE;
		return rs232ald_SendData(self, i, 
				(const byte*)&pnlI2C->i2c_wrData, wrData_lg + 6);
	    }
	}
	return false;
}

static void cbEdit_OnClick(TChkBox *self, long KeyState, bool DblClick) {
	THexEdit_SetEditMode(pnlI2C->HexEdit, self->Checked);
}

static void pnlBottom_Alg_OnResize(TWins *self) {
//  TFormMain *mf;
  int x;

//	mf = appDat.obj;
	x = self->width - pnlI2C->buRD->width - 28;
	if (x < 100) x = 100;
	SetLeft((TWins*)pnlI2C->buRD, x);
}

/*____________________________ TPnlI2C_Alg Create ________________________*/
static void TPnlI2C_Alg_CreateParams(TWins_Create_struct *wcs) {
  TPnlI2C_Alg *self;

	self = wcs->wins;
	self->subClass = (TWins*)self;
//	self->destroy = (TDESTROY)TPnlI2C_Alg__Destroy;
	self->width = 400;
	self->height = 300;
	self->Align = alClient;
//	self->BorderFu = mBorder|mBorderL;
//	self->CaptionFu = mCaptionPaint|mCaptionLeft|mCaptionTop;
// printf("%s(%d): \n", FL);
}

static TPnlI2C_Alg *TPnlI2C_Alg_Create(TWins *Parent) {
  TWins_Create_struct wcs;
  TPnlI2C_Alg *self;

	TWins_Create_struct_init(&wcs, sizeof(TPnlI2C_Alg), wtyp_TPnlI2C_Alg);
	wcs.Parent = Parent;
	wcs.CreateParams = TPnlI2C_Alg_CreateParams;
	TWins_DefaultConf(&wcs);

	self = wcs.wins;

	Set_Parent((TWins*)self, Parent);

	self->HexEdit = THexEdit_CreateA((TWins*)self, 0, 0, 200, 200, alClient);
	SetFont((TWins*)self->HexEdit, "-misc-fixed-medium-r-semicondensed--13-120-75-75-*-*-*-1");
	self->pnlBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, 0, alBottom);
	self->pnlBottom->OnResize = pnlBottom_Alg_OnResize;

	self->cbEdit = TChkBox_CreateS(self->pnlBottom, 100, 3, 44, 16, "Edit");
	self->cbEdit->OnClick = cbEdit_OnClick;
	self->buRD = TButton_CreateS((TWins*)self->pnlBottom, 300,  1, 60, 18, "RD");
	self->buRD->OnClick = buRD_OnClick;
	SetEnabled((TWins*)self->buRD, false);
	self->buWR = TButton_CreateS((TWins*)self->pnlBottom, 1, 1, 60, 18, "WR");
	self->buWR->OnClick = buWR_OnClick;
	SetEnabled((TWins*)self->buWR, false);

	self->NextTxDData = TPnlI2C_Alg_NextTxDData;
	self->OnRxDData = TPnlI2C_Alg_OnRxDData;
//	self->SetRD_heat = TPnlI2C_Alg_SetRD_heat;

	self->HexEdit->fio = TFIO_Create_MEM();
	FIO_SetMemSize(self->HexEdit->fio, 1);
	THexEdit_fioSetUp(self->HexEdit);
//	THexEdit_SetEditMode(self->HexEdit, true);

	return self;
}

/* ---------------------------------------------------------------------- */
static bool edDoCharAllow(TEdit *self, TKeyEvent *KeyEv, int idx) {
  bool res;

	res = false;
	if ((KeyEv->keysym >= '0') && (KeyEv->keysym <= '9')) res = true;
	if ((KeyEv->keysym >= 'a') && (KeyEv->keysym <= 'f')) res = true;
	if ((KeyEv->keysym >= 'A') && (KeyEv->keysym <= 'F')) res = true;
	if ((KeyEv->keysym = 'x') || (KeyEv->keysym <= 'X')) res = true;
	return res;
}

static int edI2C_size_DoKeyDown(TEdit *self, TKeyEvent *KeyEv) {
  int size;

	if ((KeyEv->keysym == XK_Return) && !self->ReadOnly && pnlI2C) {
	    if (edI2C_size_Chk(appDat.obj, &size)) {
		if (FIO_SetMemSize(pnlI2C->HexEdit->fio, size)) {
		    AdjustSize((TWins*)pnlI2C->HexEdit);
		    Refresh((TWins*)pnlI2C->HexEdit);
		}
	    }
	}
	return TEdit_DoKeyDown((TEdit*)self, KeyEv);
}

static void cbI2C_adr_OnItemSel(TComboBox *self, TMenuItem *SelItem) {
  TFormMain *mf;
  int i, max;
  char buf[16];

	i = pctoint(GetCaption_pc((TWins*)self));
	max = 16;
	if (i == 1) {
	    max = 0x100;
	} else if (i == 2) {
	    max = 0x10000;
	}

	if (pnlI2C) {
	    if (FIO_GetSize(pnlI2C->HexEdit->fio) > max) {
		FIO_SetMemSize(pnlI2C->HexEdit->fio, max);
		AdjustSize((TWins*)pnlI2C->HexEdit);
		Refresh((TWins*)pnlI2C->HexEdit);
		inttopc(buf, max);
		mf = appDat.obj;
		SetCaption((TWins*)mf->edI2C_size, buf);
	    }
	}
}

/*
typedef struct {
  struct TSegment *next;
  dword anf, end;
  const char *sdev[16];
} THwDev;
*/


static void cbI2C_Type_OnClick(TMenu *self, long KeyState, TMenuItem *mi) {
  TFormMain *mf;
  int i;
  char buf[16];

	mf = appDat.obj;
	if (mi->tag == 10) {
	    TComboBox_ClsMenu(mf->cbI2C_Dev);
//	    SetCaption((TWins*)mf->cbI2C_Dev, "---");
	    TComboBox_AddItem(mf->cbI2C_Dev, "FM24C04",  0, NULL)->tag = 0;

	    TComboBox_ClsMenu(mf->cbI2C_id);
	    SetCaption((TWins*)mf->cbI2C_id, "10");

	    TComboBox_ClsMenu(mf->cbI2C_adr);
	    SetCaption((TWins*)mf->cbI2C_adr, "1");
	    TComboBox_AddItem(mf->cbI2C_adr, "1",  0, NULL)->tag = 1;
//	    TComboBox_AddItem(mf->cbI2C_adr, "2",  0, NULL)->tag = 2;
	    return;
	}

	TComboBox_ClsMenu(mf->cbI2C_Dev);
	SetCaption((TWins*)mf->cbI2C_Dev, "---");

	TComboBox_ClsMenu(mf->cbI2C_id);
	for (i=0;i<16;i++) {
	    inttopc(buf, i);
	    TComboBox_AddItem(mf->cbI2C_id, buf,  0, NULL)->tag = i;
	}

	TComboBox_ClsMenu(mf->cbI2C_adr);
	TComboBox_AddItem(mf->cbI2C_adr, "0",  0, NULL)->tag = 0;
	TComboBox_AddItem(mf->cbI2C_adr, "1",  0, NULL)->tag = 1;
	TComboBox_AddItem(mf->cbI2C_adr, "2",  0, NULL)->tag = 2;
}

void pg_i2c_SaveFile(TFormMain *mf) {
  char *s;
  const char *pc;

	if (!pnlI2C || !pnlI2C->HexEdit->fio) return;

	pc = TpcIni_RD(appDat.appIni, APPNAME, "i2c_LastFile");
	if (pc) {
	    s = GetFilePath(pc);
	    if (s) {
		File_Dlg_SetPath(mf->FileSave_Dlg, s);
		xfree(s);
	    }
	}

	File_Dlg_SetPattern(mf->FileSave_Dlg, "*.bin", NULL);

	if (File_Dlg_Execute(mf->FileSave_Dlg)) {
	    s = NULL;
	    File_Dlg_GetEdFileName(mf->FileSave_Dlg, &s);
	    if (!FIO_Save(pnlI2C->HexEdit->fio, s)) goto err;
	    TpcIni_WR(appDat.appIni, APPNAME, "i2c_LastFile", s);
	    TpcIni_Save(appDat.appIni);
err:
	    xfree(s);
	}
}

void pg_i2c_LoadFile(TFormMain *mf) {
  int i;
  char *s;
  const char *pc;
  Tstat64 *st;
  int64_t size;
  char buf[16];

	if (!pnlI2C || !pnlI2C->HexEdit->fio) return;

	pc = TpcIni_RD(appDat.appIni, APPNAME, "i2c_LastFile");
	if (pc) {
	    s = GetFilePath(pc);
	    if (s) {
		File_Dlg_SetPath(mf->File_Dlg, s);
		xfree(s);
	    }
	}

	File_Dlg_AddPattern(mf->File_Dlg, "*.bin", NULL);

	if (File_Dlg_Execute(mf->File_Dlg)) {
	    s = NULL;
	    File_Dlg_GetEdFileName(mf->File_Dlg, &s);
	    st = file_stat_fn(s, fsFollowLink);
	    if (st->size > 0x10000) {
		appError(EMSG_ERR1, "File \"%s\" size=%d  max.=65536", GetFileName_pc(s), st->size);
		goto err;
	    }
	    if (!FIO_Load(pnlI2C->HexEdit->fio, s)) goto err;
	    TpcIni_WR(appDat.appIni, APPNAME, "i2c_LastFile", s);
	    TpcIni_Save(appDat.appIni);
	    size = FIO_GetSize(pnlI2C->HexEdit->fio);
	    inttopc(buf, size);
	    SetCaption((TWins*)mf->edI2C_size, buf);
	    i = pctoint(GetCaption_pc((TWins*)mf->cbI2C_adr));
	    if (size > 256) {
		if (i < 2) TComboBox_SetItem(mf->cbI2C_adr, 2);
	    } else if (size > 16) {
		if (i < 1) TComboBox_SetItem(mf->cbI2C_adr, 1);
	    }
err:
	    xfree(st);
	    xfree(s);
	}
}

void init_pg_i2c(TPage *self) {
  TFormMain *mf;
  int i;
  char buf[16];

	mf = appDat.obj;

	mf->pnlI2C_Alg = TWins_CreateBA((TWins*)mf->pg_i2c, 0, 0, 200, 42, 0, alTop);
	mf->lblI2C_Type = TWins_CreateLabel(mf->pnlI2C_Alg, 1, 2, 78, 18, "Type", 0, 0);
	mf->cbI2C_Type = TComboBox_CreateS(mf->pnlI2C_Alg, 1, 21, 78, 20, "Any");
	mf->cbI2C_Type->ReadOnly = true;
	TComboBox_AddItem(mf->cbI2C_Type, "Any",  0, cbI2C_Type_OnClick)->tag = -1;
	TComboBox_AddItem(mf->cbI2C_Type, "EEPROM",  0, cbI2C_Type_OnClick)->tag = 10;

	mf->lblI2C_Dev = TWins_CreateLabel(mf->pnlI2C_Alg, 83, 2, 88, 18, "Dev.", 0, 0);
	mf->cbI2C_Dev  = TComboBox_CreateS(mf->pnlI2C_Alg, 83, 21, 88, 20, "---");
	mf->cbI2C_Dev->ReadOnly = true;


	mf->lblI2C_id = TWins_CreateLabel(mf->pnlI2C_Alg, 180, 2, 82, 18, "ID / Adr", 0, 0);
	mf->cbI2C_id  = TComboBox_CreateS(mf->pnlI2C_Alg, 180, 21, 40, 20, "0");
	mf->cbI2C_id->ReadOnly = true;
	for (i=0;i<16;i++) {
	    inttopc(buf, i);
	    TComboBox_AddItem(mf->cbI2C_id, buf,  0, NULL)->tag = i;
	}

	mf->cbI2C_id_adr  = TComboBox_CreateS(mf->pnlI2C_Alg, 222, 21, 40, 20, "0");
	mf->cbI2C_id_adr->ReadOnly = true;
	for (i=0;i<8;i++) {
	    inttopc(buf, i);
	    TComboBox_AddItem(mf->cbI2C_id_adr, buf,  0, NULL)->tag = i;
	}

	mf->lblI2C_adr = TWins_CreateLabel(mf->pnlI2C_Alg, 275, 2, 120, 18, "Adr./ Adr.Byte", 0, 0);
	mf->edI2C_adr = TEdit_CreateS(mf->pnlI2C_Alg, 275, 21, 74, 20, "0x0000");

	mf->cbI2C_adr = TComboBox_CreateS(mf->pnlI2C_Alg, 350, 21, 37, 20, "0");
	mf->cbI2C_adr->ReadOnly = true;
	TComboBox_AddItem(mf->cbI2C_adr, "0",  0, NULL)->tag = 0;
	TComboBox_AddItem(mf->cbI2C_adr, "1",  0, NULL)->tag = 1;
	TComboBox_AddItem(mf->cbI2C_adr, "2",  0, NULL)->tag = 2;
	mf->cbI2C_adr->OnItemSel = cbI2C_adr_OnItemSel;

	mf->lblI2C_size = TWins_CreateLabel(mf->pnlI2C_Alg, 410, 2, 60, 18, "size", 0, 0);
	mf->edI2C_size = TEdit_CreateS(mf->pnlI2C_Alg, 410, 21, 60, 20, "1");
	mf->edI2C_size->DoCharAllow = edDoCharAllow;
	mf->edI2C_size->DoKeyDown = edI2C_size_DoKeyDown;
	
	pnlI2C = (TPnlI2C*)TPnlI2C_Alg_Create((TWins*)mf->pg_i2c);
}

void set_en_pg_i2c(TFormMain *self, bool en) {
	DoI2C = 0;
	if (!pnlI2C) return;
	if (pnlI2C->buRD) SetEnabled((TWins*)pnlI2C->buRD, en);
	if (pnlI2C->buWR) SetEnabled((TWins*)pnlI2C->buWR, en);
}

bool start_pg_i2c(TFormMain *self) {
  Trs232ald *rs232;

	if (DoI2C && pnlI2C && pnlI2C->NextTxDData) {
	    rs232 = Get_rs232();
	    if (rs232 && pnlI2C->NextTxDData(pnlI2C, rs232)) return true;
	}
	DoI2C = 0;
	return false;
}

bool OnRxDData_pg_i2c(Trs232ald *self, dword bef, const byte *data, size_t lg) {
	if (pnlI2C && pnlI2C->OnRxDData && 
	    pnlI2C->OnRxDData(self, bef, data, lg)) return true;

	DoI2C = 0;
	return false;
}
#endif /* USE_I2C_BRIDGE */
