/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include "thexfileio.h"

static void ClrData(THexFileIO *self) {
  TSegment *seg;

	ppcfree(&self->FileName);
	TLines_Clear(self->tl);

	while (self->Seg0) {
	    seg = self->Seg0;
	    self->Seg0 = seg->next;
	    xfree(seg->data);
	    xfree(seg);
	}
}

/* ______________________________________________________________________ */
static int ppctohex_lg(const char **ppc, int lg) {
  int res, Mk;
  const byte *pc;
	if (!ppc) return 0;
	pc = (const byte*)*ppc;
	if (!pc) return 0;
	Mk = res = 0;
	while (*pc && lg) {
	    if ((*pc >= '0') && (*pc <= '9')) {
		Mk |= 0x8000;
		res = (res << 4) + (*pc & 0x0f);
	    } else if (((*pc >= 'A') && (*pc <= 'F')) || 
		      ((*pc >= 'a') && (*pc <= 'f'))) {
		Mk |= 0x8000;
		res = (res << 4) + ((*pc - 7) & 0x0f);

	    } else if ((*pc == ' ') || (*pc == 0x09)) {
		lg++; //Leerzeichen erlaubt
	    } else {
		break;
	    }
	    pc++; lg--;
	}
	*ppc = (const char*)pc;
	return res;
}

/* ______________________________________________________________________ */
static int get_line_head(THexFileIO *self, const char *line, dword *padr, int *plg, const char **ppc) {
  int lg;
  dword adr;
  const char *pc;
  char c;

	adr = *padr;
	lg = *plg;
	if (*line == ':') {
	    pc = line + 1;
	    lg = ppctohex_lg(&pc, 2);
	    adr = ppctohex_lg(&pc, 4);
	    pc++;
	    c = *pc;
	    pc++;
	    if (c == '1') return 1;
	    if (c == '2') {
		self->hadr = ppctohex_lg(&pc, 1) << 16;
		return 2;
	    } else if (c == '3') {
		self->hadr = ppctohex_lg(&pc, 4) << 4; //segment *16
		self->hadr += ppctohex_lg(&pc, 4);	 //offset
		return 2;
	    } else if (c == '4') {
		self->hadr = ppctohex_lg(&pc, 4) << 16;
		return 2;

	    } else if (c != '0') {
 pc--;
 printf("%s\n", pc);
		return -1;
	    }
	    adr += self->hadr;

	} else return -1;
	if (self->max_adr < (adr + lg)) self->max_adr = adr + lg;
	*padr = adr;
	*plg = lg;
	if (ppc) *ppc = pc;
	return 0;
}

static void DebugScan(THexFileIO *self, TSegment *seg) {
 if (self->debug)
 printf("%5d   0x%08x 0x%08x\n", seg->end - seg->anf, seg->anf, seg->end);
}

static int cpf(TSegment *a, TSegment *b, int fu) {
	return a->anf - b->anf;
}

static bool scan_adr(THexFileIO *self) {
  int i, lg, ret;
  dword adr, next_adr;
  const char *line;
  TSegment *seg, **ppseg, **ppseg_s;

	self->hadr = lg = adr = next_adr = 0;
	ppseg = &self->Seg0;
	line = NULL;
	seg = NULL;
	for (i=0;i<self->tl->Count;i++) {
	    line = TLines_Get_pc(self->tl, i);
	    if (!line || !*line) continue;

	    ret = get_line_head(self, line, &adr, &lg, NULL);
	    if (ret == -1) goto endErr;
	    if (ret == 1) goto endOK;
	    if (ret == 2) continue;

	    if (seg && (next_adr != adr)) {
		seg->end = next_adr;
		DebugScan(self, seg);
		seg = NULL;
	    }
	    if (!seg) {
		seg = xcalloc(sizeof(TSegment));
		*ppseg = seg;
		ppseg = &seg->next;
		seg->anf = adr;
	    }
	    next_adr = adr + lg;
	}
endErr:
	appError(EMSG_ERR1, "Line=%d Format  %s\n", i+1, line);
	return false;
endOK:
	if (seg) seg->end = next_adr;
	DebugScan(self, seg);

//sort
	lg = 0;
	seg = self->Seg0;
	while (seg) {lg++; seg = seg->next;}
	if (lg < 2) return true;

	ppseg_s = xmalloc(sizeof(void*) * lg);
	seg = self->Seg0;
	for (i=0;i<lg;i++) {
	    ppseg_s[i] = seg;
	    seg = seg->next;
	}
	QuickSort_PL((void*)ppseg_s, 0, lg - 1, 0, (TCompareFunction)cpf);

	ppseg = &self->Seg0;
	for (i=0;i<lg;i++) {
	    seg = ppseg_s[i];
// printf("%s(%d): 0x%08x\n", FL, seg->anf);
	    seg->next = NULL;
	    *ppseg = ppseg_s[i];
	    ppseg = &seg->next;
	}
	xfree(ppseg_s);
	return true;
}

static bool chk_seg(THexFileIO *self) {
  int lg;
  dword anf, end;
  TSegment *seg, *xseg;
  bool err = false;

	seg = self->Seg0;
	while (seg) {
	    anf = seg->anf;
	    end = seg->end;
	    xseg = seg->next;
	    while (xseg) {
		if (anf < xseg->anf) {
		    if (end > xseg->anf) {
			appError(EMSG_ERR1, "Adr berlappt 0x%06x-0x%06x 0x%06x-0x%06x\n",
				anf, end, xseg->anf, xseg->end);
			err = true;
		    }
		} else {
		    if (anf < xseg->end) {
			appError(EMSG_ERR1, "Adr berlappt 0x%06x-0x%06x 0x%06x-0x%06x\n",
				anf, end, xseg->anf, xseg->end);
			err = true;
		    }
		}
		xseg = xseg->next;
	    }
	    seg = seg->next;
	}
	if (err) return false;

	seg = self->Seg0;
	while (seg) {
//	    if (seg->anf < 0x07ffff) 
	    if ((seg->anf >= self->hex_align_anf) && 
		(seg->anf < self->hex_align_end)) {
		seg->anf = (seg->anf / self->hex_align) * self->hex_align;
		seg->end = ((seg->end / self->hex_align) + 1) * self->hex_align;
	    }

//	    seg->anf = seg->anf & ~(hex_align - 1);
//	    seg->end = (seg->end + (hex_align - 1)) & ~(hex_align - 1);
	    if (seg->next && (seg->end >= seg->next->anf)) {
		seg->end = seg->next->end;
		xseg = seg->next;
		seg->next = xseg->next;
		xfree(xseg);
	    } else {
		seg = seg->next;
	    }
	}

 if (self->debug) printf("-------------- to --------------\n");
	seg = self->Seg0;
	while (seg) {
	    lg = seg->end - seg->anf;
	    if (lg > 0) {
		seg->data = xmalloc(lg);
		FillChar((char*)seg->data, lg, 0xff);
		DebugScan(self, seg);
	    }
	    seg = seg->next;
	}
	return true;
}

/* ______________________________________________________________________ */
static bool fill_seg(THexFileIO *self) {
  int lg, ret;
  dword adr, next_adr;
  const char *line, *pc;
  byte *data;
  TSegment *seg;

	self->hadr = 0;
	lg = 0;
	adr = next_adr = 0;
	for (int i=0;i<self->tl->Count;i++) {
	    line = TLines_Get_pc(self->tl, i);
	    if (!line || !*line) continue;

	    ret = get_line_head(self, line, &adr, &lg, &pc);
	    if (ret == -1) return false;
	    if (ret == 1) goto endOK;
	    if (ret == 2) continue;

// printf("0x%06x  %d  %s\n", adr, lg, pc);
	    seg = self->Seg0;
	    while (seg && (!seg->data || 
			   (adr < seg->anf)||((adr + lg) > seg->end))) {
		seg = seg->next;
	    }
	    if (!seg) {
		printf("%s(%d): Error: ???\n", FL);
		return false;
	    }

	    data = seg->data + (adr - seg->anf);
	    while (lg > 0) {
		*data = ppctohex_lg(&pc, 2);
		data++;
		lg--;
	    }
	}
	return false;
endOK:
	return true;
}

/* ______________________________________________________________________ */
static bool chk_sum(THexFileIO *self) {
  int lg, sum;
  const char *line, *pc;

	for (int i=0;i<self->tl->Count;i++) {
	    line = TLines_Get_pc(self->tl, i);
	    if (!line || !*line) continue;

	    if (*line == ':') {
		pc = line + 1;
		lg = ppctohex_lg(&pc, 2);
		sum = -lg;
		lg += 3;

	    } else {
		appError(EMSG_ERR1, "line=%d Format %s\n", i+1, line);
		return false;
	    }

	    while (lg) {
		sum -= ppctohex_lg(&pc, 2);
		lg--;
	    }
	    sum &= 0xff;
	    int tmp = ppctohex_lg(&pc, 2);
	    if (sum != tmp) {
		appError(EMSG_ERR1, "line=%d summe 0x%02x 0x%02x %s\n", i+1, sum, tmp, line);
		return false;
	    }
	}
	return true;
}

/* ______________________________________________________________________ */
bool THexFileIO_LoadFromFile(THexFileIO *self, const char *FileName) {
  dword data_size;
  char *InFile;
  TSegment *seg;
	
	ClrData(self);
	self->FileName = pcdup(FileName);
	if (!FileName || !*FileName) {
	    appError(EMSG_ERR1, "Error: File not exists \"\"\n");
	}

	InFile = path_rel2abs(FileName, NULL);
	if (!TestIsFile(InFile)) {
	    appError(EMSG_ERR1, "Error: File not exists \"%s\"\n", InFile);
ret_err:
	    xfree(InFile);
	    return false;
	}
	TLines_LoadFromFile(self->tl, InFile);

	if (!chk_sum(self)) goto ret_err;
	if (!scan_adr(self)) goto ret_err;
	if (self->hex_align) {
	    if (!chk_seg(self)) goto ret_err;
	    if (!fill_seg(self)) goto ret_err;
	}
	data_size = 0;
	seg = self->Seg0;
	while (seg) {
	    data_size += seg->end - seg->anf;
	    seg = seg->next;
	}
	return true;
}

/*_____________________________ Destroy _________________________________*/
static bool THexFileIO__Destroy(THexFileIO *self) {
	ClrData(self);
	Tfree(self->tl);
	return false;
}
/*_____________________________ Create __________________________________*/
THexFileIO *THexFileIO_Create() {
  THexFileIO *self;

	self = ttc_create(sizeof(THexFileIO), wtyp_USER);
	self->destroy = (TDESTROY)THexFileIO__Destroy;
	self->hex_align_end = 0xffffffff;
	self->tl = TLines_Create();
	return self;
}

